@@ -493,8 +493,9 @@ static inline int lump_check_opt(	struct lump *l,
 		case COND_IF_DIFF_REALMS:
 			get_ip_port_proto;
 			/* faster tests first */
-			if ((port==snd_i->send_sock->port_no) && 
+			if ((port==snd_i->send_sock->port_no) &&
 					(proto==snd_i->send_sock->proto) &&
+					(msg->rcv.proto==snd_i->proto) &&
 #ifdef USE_COMP
 					(msg->rcv.comp==snd_i->comp) &&
 #endif
@@ -512,7 +513,8 @@ static inline int lump_check_opt(	struct lump *l,
 			} else return 0;
 		case COND_IF_DIFF_PROTO:
 			get_ip_port_proto;
-			if (proto!=snd_i->send_sock->proto) {
+			if ((proto!=snd_i->send_sock->proto)
+					|| (msg->rcv.proto!=snd_i->proto)) {
 				LUMP_SET_COND_TRUE(l);
 				return 1;
 			} else return 0;
