@@ -811,7 +811,7 @@ int init_fifo_fd(char* fifo, int fifo_mode, int fifo_uid, int fifo_gid,
 	struct stat filestat;
 	int n;
 	long opt;
-	int fifo_read;
+	int fifo_read = -1;
 	
 	if (fifo == NULL) {
 		ERR("null fifo: no fifo will be opened\n");
@@ -819,11 +819,10 @@ int init_fifo_fd(char* fifo, int fifo_mode, int fifo_uid, int fifo_gid,
 		return -1;
 	}
 	if (strlen(fifo) == 0) {
-		ERR("emtpy fifo: fifo disabled\n");
+		ERR("empty fifo: fifo disabled\n");
 		return -1;
 	}
 	
-	
 	DBG("Opening fifo...\n");
 	n = stat(fifo, &filestat);
 	if (n == 0) {
@@ -862,26 +861,29 @@ int init_fifo_fd(char* fifo, int fifo_mode, int fifo_uid, int fifo_gid,
 	
 	fifo_read = open(fifo, O_RDONLY | O_NONBLOCK, 0);
 	if (fifo_read < 0) {
-		ERR("fifo_read did not open: %s\n",
-		    strerror(errno));
+		ERR("fifo_read did not open: %s\n", strerror(errno));
 		return -1;
 	}
 	/* make sure the read fifo will not close */
 	*fifo_write = open(fifo, O_WRONLY | O_NONBLOCK, 0);
 	if (*fifo_write < 0) {
-		ERR("fifo_write did not open: %s\n",
-		    strerror(errno));
+		ERR("fifo_write did not open: %s\n", strerror(errno));
+		close(fifo_read);
 		return -1;
 	}
 	/* set read fifo blocking mode */
 	if ((opt = fcntl(fifo_read, F_GETFL)) == -1) {
-		ERR("fcntl(F_GETFL) failed: %s [%d]\n",
-		    strerror(errno), errno);
+		ERR("fcntl(F_GETFL) failed: %s [%d]\n", strerror(errno), errno);
+		close(fifo_read);
+		close(*fifo_write);
+		*fifo_write = -1;
 		return -1;
 	}
 	if (fcntl(fifo_read, F_SETFL, opt & (~O_NONBLOCK)) == -1) {
-		ERR("fcntl(F_SETFL) failed: %s [%d]\n",
-		    strerror(errno), errno);
+		ERR("fcntl(F_SETFL) failed: %s [%d]\n", strerror(errno), errno);
+		close(fifo_read);
+		close(*fifo_write);
+		*fifo_write = -1;
 		return -1;
 	}
 	return fifo_read;
@@ -1090,6 +1092,7 @@ static int print_value(rpc_ctx_t* ctx, char fmt, va_list* ap)
 	str str_val;
 	str* sp;
 	char buf[256];
+	str null_value = str_init("<null string>");
 
 	switch(fmt) {
 	case 'd':
@@ -1102,7 +1105,7 @@ static int print_value(rpc_ctx_t* ctx, char fmt, va_list* ap)
 			goto err;
 		}
 		break;
-		
+
 	case 'f':
 		str_val.s = buf;
 		str_val.len = snprintf(buf, 256, "%f", va_arg(*ap, double));
@@ -1118,39 +1121,48 @@ static int print_value(rpc_ctx_t* ctx, char fmt, va_list* ap)
 			goto err;
 		}
 		break;
-		
+
 	case 'b':
 		str_val.len = 1;
 		str_val.s = ((va_arg(*ap, int) == 0) ? "0" : "1");
 		l = new_chunk(&str_val);
 		if (!l) {
-			rpc_fault(ctx, 500, "Internal Server Error, line %d", 
+			rpc_fault(ctx, 500, "Internal Server Error, line %d",
 						ctx->line_no);
 			goto err;
 		}
 		break;
-				
+
 	case 's':
 		str_val.s = va_arg(*ap, char*);
-		str_val.len = strlen(str_val.s);
+		if(str_val.s!=NULL) {
+			str_val.len = strlen(str_val.s);
+		} else {
+			str_val = null_value;
+		}
 		l = new_chunk_escape(&str_val, 0);
 		if (!l) {
-			rpc_fault(ctx, 500, "Internal Server Error, line %d", 
+			rpc_fault(ctx, 500, "Internal Server Error, line %d",
 						ctx->line_no);
 			goto err;
 		}
 		break;
-		
+
 	case 'S':
 		sp = va_arg(*ap, str*);
-		l = new_chunk_escape(sp, 0);
+		if(sp!=NULL && sp->s!=NULL) {
+			str_val = *sp;
+		} else {
+			str_val = null_value;
+		}
+		l = new_chunk_escape(&str_val, 0);
 		if (!l) {
-			rpc_fault(ctx, 500, "Internal Server Error, line %d", 
+			rpc_fault(ctx, 500, "Internal Server Error, line %d",
 							ctx->line_no);
 			goto err;
 		}
 		break;
-		
+
 	default:
 		rpc_fault(ctx, 500, "Bug In SER (Invalid formatting character %c)", fmt);
 		ERR("Invalid formatting character\n");
@@ -1202,11 +1214,12 @@ static int rpc_struct_printf(struct text_chunk* c, char* name, char* fmt, ...)
 {
 	int n, buf_size;
 	char* buf;
+	char* buf0;
 	va_list ap;
 	str s, nm;
 	struct text_chunk* l, *m;
 	rpc_ctx_t* ctx;
-	
+
 	ctx=(rpc_ctx_t*)c->ctx;
 	buf = (char*)ctl_malloc(RPC_BUF_SIZE);
 	if (!buf) {
@@ -1214,7 +1227,7 @@ static int rpc_struct_printf(struct text_chunk* c, char* name, char* fmt, ...)
 		ERR("No memory left\n");
 		return -1;
 	}
-	
+
 	buf_size = RPC_BUF_SIZE;
 	while (1) {
 		     /* Try to print in the allocated space. */
@@ -1240,7 +1253,7 @@ static int rpc_struct_printf(struct text_chunk* c, char* name, char* fmt, ...)
 				ERR("Error while creating text_chunk structure");
 				goto err;
 			}
-			
+
 			l->flags |= CHUNK_MEMBER_VALUE;
 			l->next = c->next;
 			c->next = l;
@@ -1258,11 +1271,12 @@ static int rpc_struct_printf(struct text_chunk* c, char* name, char* fmt, ...)
 		} else {          /* glibc 2.0 */
 			buf_size *= 2;  /* twice the old size */
 		}
-		if ((buf = ctl_realloc(buf, buf_size)) == 0) {
+		if ((buf0 = ctl_realloc(buf, buf_size)) == 0) {
 			rpc_fault(ctx, 500, "Internal Server Error (No memory left)");
 			ERR("No memory left\n");
 			goto err;
 		}
+		buf = buf0;
 	}
 	return 0;
  err:
@@ -1275,6 +1289,7 @@ static int rpc_rpl_printf(rpc_ctx_t* ctx, char* fmt, ...)
 {
 	int n, buf_size;
 	char* buf;
+	char* buf0;
 	va_list ap;
 	str s;
 	struct text_chunk* l;
@@ -1285,7 +1300,7 @@ static int rpc_rpl_printf(rpc_ctx_t* ctx, char* fmt, ...)
 		ERR("No memory left\n");
 		return -1;
 	}
-	
+
 	buf_size = RPC_BUF_SIZE;
 	while (1) {
 		     /* Try to print in the allocated space. */
@@ -1312,11 +1327,12 @@ static int rpc_rpl_printf(rpc_ctx_t* ctx, char* fmt, ...)
 		} else {          /* glibc 2.0 */
 			buf_size *= 2;  /* twice the old size */
 		}
-		if ((buf = ctl_realloc(buf, buf_size)) == 0) {
+		if ((buf0 = ctl_realloc(buf, buf_size)) == 0) {
 			rpc_fault(ctx, 500, "Internal Server Error (No memory left)");
 			ERR("No memory left\n");
 			goto err;
 		}
+		buf = buf0;
 	}
 	return 0;
  err:
@@ -1362,7 +1378,7 @@ static int rpc_scan(rpc_ctx_t* ctx, char* fmt, ...)
 		case 'd': /* Integer */
 			if (!line.len) {
 				if(nofault==0)
-					rpc_fault(ctx, 400, "Invalid parameter value on line %d", 
+					rpc_fault(ctx, 400, "Invalid parameter value on line %d",
 									ctx->line_no);
 				goto error;
 			}
@@ -1373,14 +1389,14 @@ static int rpc_scan(rpc_ctx_t* ctx, char* fmt, ...)
 		case 'f': /* double */
 			if (!line.len) {
 				if(nofault==0)
-					rpc_fault(ctx, 400, "Invalid parameter value on line %d", 
+					rpc_fault(ctx, 400, "Invalid parameter value on line %d",
 								ctx->line_no);
 				goto error;
 			}
 			double_ptr = va_arg(ap, double*);
 			*double_ptr = strtod(line.s, 0);
 			break;
-			
+
 		case 's': /* zero terminated string */
 		case 'S': /* str structure */
 			l = new_chunk_unescape(&line);
@@ -1438,6 +1454,7 @@ static int rpc_struct_add(struct text_chunk* s, char* fmt, ...)
 	va_list ap;
 	struct text_chunk* m, *c;
 	rpc_ctx_t* ctx;
+	str null_value = str_init("<null string>");
 
 	ctx=(rpc_ctx_t*)s->ctx;
 	va_start(ap, fmt);
@@ -1451,7 +1468,7 @@ static int rpc_struct_add(struct text_chunk* s, char* fmt, ...)
 			goto err;
 		}
 		m->flags |= CHUNK_MEMBER_NAME;
-		
+
 		if(*fmt=='{' || *fmt=='[') {
 			void_ptr = va_arg(ap, void**);
 			m->ctx=ctx;
@@ -1484,13 +1501,22 @@ static int rpc_struct_add(struct text_chunk* s, char* fmt, ...)
 
 			case 's':
 				st.s = va_arg(ap, char*);
-				st.len = strlen(st.s);
+				if(st.s==NULL) {
+					st = null_value;
+				} else {
+					st.len = strlen(st.s);
+				}
 				c = new_chunk_escape(&st, 1);
 				break;
 
 			case 'S':
 				sp = va_arg(ap, str*);
-				c = new_chunk_escape(sp, 1);
+				if(sp!=NULL && sp->s!=NULL) {
+					st = *sp;
+				} else {
+					st = null_value;
+				}
+				c = new_chunk_escape(&st, 1);
 				break;
 
 			default:
@@ -1570,7 +1596,7 @@ static int rpc_struct_scan(struct rpc_struct* s, char* fmt, ...)
 			va_end(ap);
 			return read;
 		}
-		
+
 		switch(*fmt) {
 		case 'b': /* Bool */
 		case 't': /* Date and time */
@@ -1593,7 +1619,7 @@ static int rpc_struct_scan(struct rpc_struct* s, char* fmt, ...)
 			     /* String in text_chunk is always zero terminated */
 			*double_ptr = strtod(val->s.s, 0);
 			break;
-			
+
 		case 's': /* zero terminated string */
 			char_ptr = va_arg(ap, char**);
 			     /* String in text_chunk is always zero terminated */
@@ -1602,7 +1628,6 @@ static int rpc_struct_scan(struct rpc_struct* s, char* fmt, ...)
 
 		case 'S': /* str structure */
 			str_ptr = va_arg(ap, str*);
-			str_ptr->len = strlen(str_ptr->s);
 			*str_ptr = val->s;
 			break;
 		default:
