@@ -50,6 +50,7 @@
 #include "../../core/pvar.h"
 #include "../../core/cfg/cfg_struct.h"
 #include "../../core/fmsg.h"
+#include "../../core/kemi.h"
 
 #include "../../modules/tm/tm_load.h"
 #include "../../modules/pv/pv_api.h"
@@ -65,14 +66,18 @@ extern unsigned int q_idx;
 extern char q_id[MAX_ID_LEN+1];
 
 int http_timeout = 500; /* query timeout in ms */
+int tcp_keepalive = 0; /* TCP keepalives (default disabled) */
+int tcp_ka_idle = 0; /* TCP keep-alive idle time wait */
+int tcp_ka_interval = 0; /* TCP keep-alive interval */
 int hash_size = 2048;
 int tls_version = 0; // Use default SSL version in HTTPS requests (see curl/curl.h)
 int tls_verify_host = 1; // By default verify host in HTTPS requests
 int tls_verify_peer = 1; // By default verify peer in HTTPS requests
 int curl_verbose = 0;
-str tls_client_cert = STR_STATIC_INIT(""); // client SSL certificate path, defaults to NULL
-str tls_client_key = STR_STATIC_INIT(""); // client SSL certificate key path, defaults to NULL
-str tls_ca_path = STR_STATIC_INIT(""); // certificate authority dir path, defaults to NULL
+int curl_follow_redirect = 0;
+char* tls_client_cert = NULL; // client SSL certificate path, defaults to NULL
+char* tls_client_key = NULL; // client SSL certificate key path, defaults to NULL
+char* tls_ca_path = NULL; // certificate authority dir path, defaults to NULL
 static char *memory_manager = "shm";
 extern int curl_memory_manager;
 unsigned int default_authmethod = CURLAUTH_BASIC | CURLAUTH_DIGEST;
@@ -83,7 +88,6 @@ static void mod_destroy(void);
 
 static int w_http_async_query(sip_msg_t* msg, char* query, char* rt);
 static int set_query_param(str* param, str input);
-static int fixup_http_async_query(void** param, int param_no);
 
 /* pv api binding */
 static int ah_get_reason(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
@@ -95,6 +99,7 @@ static int ah_get_body_size(struct sip_msg *msg, pv_param_t *param, pv_value_t *
 static int ah_get_msg_buf(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
 static int ah_get_msg_len(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
 static int ah_parse_req_name(pv_spec_p sp, str *in);
+static int ah_parse_time_name(pv_spec_p sp, str *in);
 static int ah_set_req(struct sip_msg* msg, pv_param_t *param, int op, pv_value_t *val);
 static int ah_get_id(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
 
@@ -104,6 +109,7 @@ static str pv_str_0 = {"0", 1};
 
 static int ah_get_ok(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
 static int ah_get_err(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
+static int ah_get_time(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
 
 /* tm */
 struct tm_binds tmb;
@@ -121,11 +127,18 @@ enum http_req_name_t {
 	E_HRN_TLS_CA_PATH, E_HRN_TLS_CLIENT_KEY,
 	E_HRN_TLS_CLIENT_CERT, E_HRN_SUSPEND,
 	E_HRN_BODY, E_HRN_AUTHMETHOD, E_HRN_USERNAME,
-	E_HRN_PASSWORD
+	E_HRN_PASSWORD, E_HRN_TCP_KA, E_HRN_TCP_KA_IDLE,
+	E_HRN_TCP_KA_INTERVAL, E_HRN_FOLLOW_REDIRECT
+};
+
+enum http_time_name_t {
+	E_HT_TOTAL = 0,
+	E_HT_LOOKUP, E_HT_CONNECT, E_HT_REDIRECT,
+	E_HT_APPCONNECT, E_HT_PRETRANSFER, E_HT_STARTTRANSFER
 };
 
 static cmd_export_t cmds[]={
-	{"http_async_query",  (cmd_function)w_http_async_query, 2, fixup_http_async_query,
+	{"http_async_query",  (cmd_function)w_http_async_query, 2, fixup_spve_spve,
 		0, ANY_ROUTE},
 	{0, 0, 0, 0, 0, 0}
 };
@@ -138,11 +151,15 @@ static param_export_t params[]={
 	{"tls_verify_host",		INT_PARAM,		&tls_verify_host},
 	{"tls_verify_peer",		INT_PARAM,		&tls_verify_peer},
 	{"curl_verbose",		INT_PARAM,		&curl_verbose},
-	{"tls_client_cert",		PARAM_STR,		&tls_client_cert},
-	{"tls_client_key",		PARAM_STR,		&tls_client_key},
-	{"tls_ca_path",			PARAM_STR,		&tls_ca_path},
+	{"curl_follow_redirect",	INT_PARAM,		&curl_follow_redirect},
+	{"tls_client_cert",		PARAM_STRING,	&tls_client_cert},
+	{"tls_client_key",		PARAM_STRING,	&tls_client_key},
+	{"tls_ca_path",			PARAM_STRING,	&tls_ca_path},
 	{"memory_manager",		PARAM_STRING,	&memory_manager},
 	{"authmethod",			PARAM_INT,		&default_authmethod },
+	{"tcp_keepalive",	    INT_PARAM,		&tcp_keepalive},
+	{"tcp_ka_idle",	        INT_PARAM,		&tcp_ka_idle},
+	{"tcp_ka_interval",	    INT_PARAM,		&tcp_ka_interval},
 	{0, 0, 0}
 };
 
@@ -183,6 +200,9 @@ static pv_export_t pvs[] = {
 	{STR_STATIC_INIT("http_err"),
 		PVT_OTHER, ah_get_err, 0,
 		0, 0, 0, 0},
+	{STR_STATIC_INIT("http_time"),
+		PVT_OTHER, ah_get_time, 0,
+		ah_parse_time_name, 0, 0, 0},
 	{STR_STATIC_INIT("http_req"),
 		PVT_OTHER, pv_get_null, ah_set_req,
 		ah_parse_req_name, 0, 0, 0},
@@ -193,34 +213,19 @@ static pv_export_t pvs[] = {
 };
 
 struct module_exports exports = {
-	"http_async_client",
-	DEFAULT_DLFLAGS, /* dlopen flags */
-	cmds,
-	params,
-	mod_stats,   	/* exported statistics */
-	0,              /* exported MI functions */
-	pvs,            /* exported pseudo-variables */
-	0,              /* extra processes */
-	mod_init,       /* module initialization function */
-	0,              /* response function */
-	mod_destroy,    /* destroy function */
-	child_init      /* per child init function */
+	"http_async_client",	/* module name */
+	DEFAULT_DLFLAGS,		/* dlopen flags */
+	cmds,					/* exported functions */
+	params,					/* exported parameters */
+	0,						/* RPC method exports */
+	pvs,					/* exported pseudo-variables */
+	0,						/* response handling function */
+	mod_init,				/* module initialization function */
+	child_init,				/* per-child init function */
+	mod_destroy				/* module destroy function */
 };
 
 
-int mod_register(char *path, int *dlflags, void *p1, void *p2)
-{
-	pv_register_api_t pvra;
-
-	pvra = (pv_register_api_t)find_export("pv_register_api", NO_SCRIPT, 0);
-	if (!pvra) {
-		LM_ERR("Cannot import pv functions (pv module must be loaded before this module)\n");
-		return -1;
-	}
-	pvra(&pv_api);
-	return 0;
-}
-
 /**
  * init module function
  */
@@ -264,6 +269,13 @@ static int mod_init(void)
 	tls_verify_host = tls_verify_host?1:0;
 	tls_verify_peer = tls_verify_peer?1:0;
 
+	if (tcp_keepalive) {
+		LM_INFO("TCP keepalives enabled\n");
+	}
+	/* check tcp keepalive parameters */
+	if ((tcp_ka_idle > 0 || tcp_ka_interval > 0) && !(tcp_keepalive > 0)) {
+		LM_WARN("either 'tcp_ka_idle' or 'tcp_ka_interval' are set but 'tcp_keepalive' is disabled: they will be ignored\n");
+	}
 	/* init http parameters list */
 	init_query_params(&ah_params);
 
@@ -276,6 +288,8 @@ static int mod_init(void)
 		return -1;
 	}
 
+	set_curl_mem_callbacks();
+
 	/* init faked sip msg */
 	if(faked_msg_init()<0) {
 		LM_ERR("failed to init faked sip msg\n");
@@ -346,7 +360,7 @@ static int child_init(int rank)
 			LM_ERR("failed to initialize worker process: %d\n", i);
 			return -1;
 		}
-		pid=fork_process(PROC_RPC, "Http Worker", 1);
+		pid=fork_process(PROC_RPC, "Http Async Worker", 1);
 		if (pid<0)
 			return -1; /* error */
 		if(pid==0) {
@@ -384,9 +398,7 @@ static void mod_destroy(void)
 static int w_http_async_query(sip_msg_t *msg, char *query, char* rt)
 {
 	str sdata;
-	cfg_action_t *act;
 	str rn;
-	int ri;
 
 	if(msg==NULL)
 		return -1;
@@ -405,22 +417,29 @@ static int w_http_async_query(sip_msg_t *msg, char *query, char* rt)
 		LM_ERR("no route block name\n");
 		return -1;
 	}
-
-	ri = route_get(&main_rt, rn.s);
-	if(ri<0)
-	{
-		LM_ERR("unable to find route block [%.*s]\n", rn.len, rn.s);
+	if(rn.s==NULL || rn.len == 0) {
+		LM_ERR("invalid route name parameter\n");
 		return -1;
 	}
-	act = main_rt.rlist[ri];
-	if(act==NULL)
-	{
-		LM_ERR("empty action lists in route block [%.*s]\n", rn.len, rn.s);
+	return async_send_query(msg, &sdata, &rn);
+}
+
+/**
+ *
+ */
+static int ki_http_async_query(sip_msg_t *msg, str *sdata, str *rn)
+{
+	if(msg==NULL)
+		return -1;
+	if(sdata==NULL || sdata->len <= 0) {
+		LM_ERR("invalid data parameter\n");
 		return -1;
 	}
-
-	return async_send_query(msg, &sdata, act);
-
+	if(rn->s==NULL || rn->len <= 0) {
+		LM_ERR("invalid route name parameter\n");
+		return -1;
+	}
+	return async_send_query(msg, sdata, rn);
 }
 
 #define _IVALUE_ERROR(NAME) LM_ERR("invalid parameter '" #NAME "' (must be a number)\n")
@@ -479,22 +498,6 @@ static int set_query_cparam(char** param, str input)
 	return 1;
 }
 
-/**
- *
- */
-static int fixup_http_async_query(void** param, int param_no)
-{
-	if (param_no == 1) {
-		return fixup_spve_null(param, 1);
-	}
-	if (param_no == 2) {
-		return fixup_var_str_12(param, param_no);
-	}
-
-	LM_ERR("invalid parameter number <%d>\n", param_no);
-	return -1;
-}
-
 /* module PVs */
 
 #define AH_WRAP_GET_PV(AH_F, PV_F) static int AH_F(struct sip_msg *msg, pv_param_t *param, pv_value_t *res) \
@@ -554,6 +557,31 @@ static int ah_get_err(struct sip_msg *msg, pv_param_t *param, pv_value_t *res) {
 	}
 }
 
+static int ah_get_time(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res)
+{
+	if(msg==NULL || param==NULL)
+		return -1;
+
+	switch(param->pvn.u.isname.name.n)
+	{
+		case E_HT_LOOKUP:
+			return pv_get_uintval(msg, param, res, ah_time.lookup);
+		case E_HT_CONNECT:
+			return pv_get_uintval(msg, param, res, ah_time.connect);
+		case E_HT_REDIRECT:
+			return pv_get_uintval(msg, param, res, ah_time.redirect);
+		case E_HT_APPCONNECT:
+			return pv_get_uintval(msg, param, res, ah_time.appconnect);
+		case E_HT_PRETRANSFER:
+			return pv_get_uintval(msg, param, res, ah_time.pretransfer);
+		case E_HT_STARTTRANSFER:
+			return pv_get_uintval(msg, param, res, ah_time.starttransfer);
+		default:
+			return pv_get_uintval(msg, param, res, ah_time.total);
+	}
+}
+
 static int ah_parse_req_name(pv_spec_p sp, str *in) {
 	if(sp==NULL || in==NULL || in->len<=0)
 		return -1;
@@ -582,6 +610,8 @@ static int ah_parse_req_name(pv_spec_p sp, str *in) {
 				sp->pvp.pvn.u.isname.name.n = E_HRN_TIMEOUT;
 			else if(strncmp(in->s, "suspend", 7)==0)
 				sp->pvp.pvn.u.isname.name.n = E_HRN_SUSPEND;
+			else if(strncmp(in->s, "ka-idle", 7)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HRN_TCP_KA_IDLE;
 			else goto error;
 			break;
 		case 8:
@@ -591,6 +621,11 @@ static int ah_parse_req_name(pv_spec_p sp, str *in) {
 				sp->pvp.pvn.u.isname.name.n = E_HRN_PASSWORD;
 			else goto error;
 			break;
+		case 9:
+			if(strncmp(in->s, "keepalive", 9)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HRN_TCP_KA;
+			else goto error;
+			break;
 		case 10:
 			if(strncmp(in->s, "authmethod", 10)==0)
 				sp->pvp.pvn.u.isname.name.n = E_HRN_AUTHMETHOD;
@@ -599,6 +634,8 @@ static int ah_parse_req_name(pv_spec_p sp, str *in) {
 		case 11:
 			if(strncmp(in->s, "tls_ca_path", 11)==0)
 				sp->pvp.pvn.u.isname.name.n = E_HRN_TLS_CA_PATH;
+			else if(strncmp(in->s, "ka-interval", 11)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HRN_TCP_KA_INTERVAL;
 			else goto error;
 			break;
 		case 14:
@@ -609,6 +646,8 @@ static int ah_parse_req_name(pv_spec_p sp, str *in) {
 		case 15:
 			if(strncmp(in->s, "tls_client_cert", 15)==0)
 				sp->pvp.pvn.u.isname.name.n = E_HRN_TLS_CLIENT_CERT;
+			else if(strncmp(in->s, "follow_redirect", 15)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HRN_FOLLOW_REDIRECT;
 			else goto error;
 			break;
 		default:
@@ -624,6 +663,60 @@ error:
 	return -1;
 }
 
+static int ah_parse_time_name(pv_spec_p sp, str *in) {
+	if(sp==NULL || in==NULL || in->len<0)
+		return -1;
+
+	switch(in->len)
+	{
+		case 5:
+			if(strncmp(in->s, "total", 5)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HT_TOTAL;
+			else goto error;
+			break;
+		case 6:
+			if(strncmp(in->s, "lookup", 6)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HT_LOOKUP;
+			else goto error;
+			break;
+		case 7:
+			if(strncmp(in->s, "connect", 7)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HT_CONNECT;
+			else goto error;
+			break;
+		case 8:
+			if(strncmp(in->s, "redirect", 8)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HT_REDIRECT;
+			else goto error;
+			break;
+		case 10:
+			if(strncmp(in->s, "appconnect", 10)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HT_APPCONNECT;
+			else goto error;
+			break;
+		case 11:
+			if(strncmp(in->s, "pretransfer", 11)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HT_PRETRANSFER;
+			else goto error;
+			break;
+		case 13:
+			if(strncmp(in->s, "starttransfer", 13)==0)
+				sp->pvp.pvn.u.isname.name.n = E_HT_STARTTRANSFER;
+			else goto error;
+			break;
+		default:
+			goto error;
+	}
+	sp->pvp.pvn.type = PV_NAME_INTSTR;
+	sp->pvp.pvn.u.isname.type = 0;
+
+	return 0;
+
+error:
+	LM_ERR("unknown http_time name %.*s\n", in->len, in->s);
+	return -1;
+}
+
 static int ah_set_req(struct sip_msg* msg, pv_param_t *param,
 		int op, pv_value_t *val)
 {
@@ -679,9 +772,7 @@ static int ah_set_req(struct sip_msg* msg, pv_param_t *param,
 				LM_ERR("invalid value type for $http_req(tls_ca_path)\n");
 				return -1;
 			}
-			set_query_param(&ah_params.tls_ca_path, tval->rs);
-		} else {
-			set_query_param(&ah_params.tls_ca_path, tls_ca_path);
+			set_query_cparam(&ah_params.tls_ca_path, tval->rs);
 		}
 		break;
 	case E_HRN_TLS_CLIENT_KEY:
@@ -690,9 +781,7 @@ static int ah_set_req(struct sip_msg* msg, pv_param_t *param,
 				LM_ERR("invalid value type for $http_req(tls_client_key)\n");
 				return -1;
 			}
-			set_query_param(&ah_params.tls_client_key, tval->rs);
-		} else {
-			set_query_param(&ah_params.tls_client_key, tls_client_key);
+			set_query_cparam(&ah_params.tls_client_key, tval->rs);
 		}
 		break;
 	case E_HRN_TLS_CLIENT_CERT:
@@ -701,9 +790,7 @@ static int ah_set_req(struct sip_msg* msg, pv_param_t *param,
 				LM_ERR("invalid value type for $http_req(tls_client_cert)\n");
 				return -1;
 			}
-			set_query_param(&ah_params.tls_client_cert, tval->rs);
-		} else {
-			set_query_param(&ah_params.tls_client_cert, tls_client_cert);
+			set_query_cparam(&ah_params.tls_client_cert, tval->rs);
 		}
 		break;
 	case E_HRN_SUSPEND:
@@ -755,7 +842,82 @@ static int ah_set_req(struct sip_msg* msg, pv_param_t *param,
 			set_query_cparam(&ah_params.password, tval->rs);
 		}
 		break;
+	case E_HRN_TCP_KA:
+		if (tval) {
+			if (!(tval->flags & PV_VAL_INT)) {
+				LM_ERR("invalid value type for $http_req(keepalive)\n");
+				return -1;
+			}
+			ah_params.tcp_keepalive = tval->ri;
+		} else {
+			ah_params.tcp_keepalive = tcp_keepalive;
+		}
+		break;
+
+	case E_HRN_TCP_KA_IDLE:
+		if (tval) {
+			if (!(tval->flags & PV_VAL_INT)) {
+				LM_ERR("invalid value type for $http_req(ka-idle)\n");
+				return -1;
+			}
+			ah_params.tcp_ka_idle = tval->ri;
+		} else {
+			ah_params.tcp_ka_idle = tcp_ka_idle;
+		}
+		break;
+
+	case E_HRN_TCP_KA_INTERVAL:
+		if (tval) {
+			if (!(tval->flags & PV_VAL_INT)) {
+				LM_ERR("invalid value type for $http_req(ka-interval)\n");
+				return -1;
+			}
+			ah_params.tcp_ka_interval = tval->ri;
+		} else {
+			ah_params.tcp_ka_interval = tcp_ka_interval;
+		}
+		break;
+  case E_HRN_FOLLOW_REDIRECT:
+		if (tval) {
+			if (!(tval->flags & PV_VAL_INT)) {
+				LM_ERR("invalid value type for $http_req(follow_redirect)\n");
+				return -1;
+			}
+			ah_params.follow_redirect = tval->ri?1:0;
+		} else {
+			ah_params.follow_redirect = curl_follow_redirect;
+		}
+		break;
 	}
 
 	return 1;
 }
+
+/**
+ *
+ */
+/* clang-format off */
+static sr_kemi_t sr_kemi_http_async_client_exports[] = {
+	{ str_init("http_async_client"), str_init("query"),
+		SR_KEMIP_INT, ki_http_async_query,
+		{ SR_KEMIP_STR, SR_KEMIP_STR, SR_KEMIP_NONE,
+			SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
+	},
+
+	{ {0, 0}, {0, 0}, 0, NULL, { 0, 0, 0, 0, 0, 0 } }
+};
+/* clang-format on */
+
+int mod_register(char *path, int *dlflags, void *p1, void *p2)
+{
+	pv_register_api_t pvra;
+
+	pvra = (pv_register_api_t)find_export("pv_register_api", NO_SCRIPT, 0);
+	if (!pvra) {
+		LM_ERR("Cannot import pv functions (pv module must be loaded before this module)\n");
+		return -1;
+	}
+	pvra(&pv_api);
+	sr_kemi_modules_add(sr_kemi_http_async_client_exports);	
+	return 0;
+}
