@@ -46,7 +46,7 @@
 #include "../../core/receive.h"
 #include "../../core/action.h"
 #include "../../core/script_cb.h"
-
+#include "../../core/kemi.h"
 
 #include "kz_amqp.h"
 #include "kz_json.h"
@@ -71,6 +71,7 @@ extern int kz_cmd_pipe;
 extern struct timeval kz_amqp_tv;
 extern struct timeval kz_qtimeout_tv;
 extern struct timeval kz_timer_tv;
+extern struct timeval kz_amqp_connect_timeout_tv;
 
 extern str kz_amqps_ca_cert;
 extern str kz_amqps_cert;
@@ -80,6 +81,9 @@ extern int kz_amqps_verify_hostname;
 
 extern pv_spec_t kz_query_timeout_spec;
 
+extern int kazoo_kemi_enabled;
+extern str kazoo_event_callback;
+
 const amqp_bytes_t kz_amqp_empty_bytes = { 0, NULL };
 const amqp_table_t kz_amqp_empty_table = { 0, NULL };
 
@@ -139,7 +143,7 @@ static inline str* kz_str_dup(str* src)
 {
 	char *dst_char = (char*)shm_malloc(sizeof(str)+src->len+1);
 	if (!dst_char) {
-		LM_ERR("error allocating shared memory for str");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 	str* dst = (str*)dst_char;
@@ -156,7 +160,7 @@ static inline str* kz_str_dup_from_char(char* src)
 	int len = strlen(src);
 	char *dst_char = (char*)shm_malloc(sizeof(str)+len+1);
 	if (!dst_char) {
-		LM_ERR("error allocating shared memory for str");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 	str* dst = (str*)dst_char;
@@ -215,7 +219,7 @@ static inline str* kz_local_str_dup(str* src)
 {
 	char *dst_char = (char*)pkg_malloc(sizeof(str)+src->len+1);
 	if (!dst_char) {
-		LM_ERR("error allocating shared memory for str");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 	str* dst = (str*)dst_char;
@@ -246,7 +250,7 @@ static inline str* kz_str_from_amqp_bytes(amqp_bytes_t src)
 {
 	char *dst_char = (char*)shm_malloc(sizeof(str)+src.len+1);
 	if (!dst_char) {
-		LM_ERR("error allocating shared memory for str");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 	str* dst = (str*)dst_char;
@@ -679,7 +683,7 @@ void kz_amqp_destroy() {
 	kz_hash_destroy();
 }
 
-#define KZ_URL_MAX_SIZE 100
+#define KZ_URL_MAX_SIZE 512
 static char* KZ_URL_ROOT = "/";
 
 int kz_amqp_add_connection(modparam_t type, void* val)
@@ -841,7 +845,7 @@ int kz_amqp_connection_open_ssl(kz_amqp_conn_ptr rmq) {
     amqp_ssl_socket_set_verify_hostname(rmq->socket, kz_amqps_verify_hostname);
 #endif
 
-    if (amqp_socket_open(rmq->socket, rmq->server->connection->info.host, rmq->server->connection->info.port)) {
+    if (amqp_socket_open_noblock(rmq->socket, rmq->server->connection->info.host, rmq->server->connection->info.port, &kz_amqp_connect_timeout_tv)) {
 	LM_ERR("Failed to open SSL socket to AMQP broker : %s : %i\n",
 	rmq->server->connection->info.host, rmq->server->connection->info.port);
 	goto nosocket;
@@ -895,7 +899,7 @@ int kz_amqp_connection_open(kz_amqp_conn_ptr rmq) {
     	goto nosocket;
     }
 
-    if (amqp_socket_open(rmq->socket, rmq->server->connection->info.host, rmq->server->connection->info.port)) {
+    if (amqp_socket_open_noblock(rmq->socket, rmq->server->connection->info.host, rmq->server->connection->info.port, &kz_amqp_connect_timeout_tv)) {
     	LM_DBG("Failed to open TCP socket to AMQP broker\n");
     	goto nosocket;
     }
@@ -1212,9 +1216,9 @@ int kz_amqp_pipe_send_receive(str *str_exchange, str *str_routing_key, str *str_
     return ret;
 }
 
-int kz_amqp_publish(struct sip_msg* msg, char* exchange, char* routing_key, char* payload)
+int kz_amqp_publish_ex(struct sip_msg* msg, char* exchange, char* routing_key, char* payload, char* _pub_flags)
 {
-	  str json_s;
+	  str pl_s;
 	  str exchange_s;
 	  str routing_key_s;
 
@@ -1228,25 +1232,37 @@ int kz_amqp_publish(struct sip_msg* msg, char* exchange, char* routing_key, char
 			return -1;
 		}
 
-		if (fixup_get_svalue(msg, (gparam_p)payload, &json_s) != 0) {
+		if (fixup_get_svalue(msg, (gparam_p)payload, &pl_s) != 0) {
 			LM_ERR("cannot get json string value : %s\n", payload);
 			return -1;
 		}
 
-		struct json_object *j = json_tokener_parse(json_s.s);
-
-		if (is_error(j)) {
-			LM_ERR("empty or invalid JSON payload : %.*s\n", json_s.len, json_s.s);
+		if (routing_key_s.len > MAX_ROUTING_KEY_SIZE) {
+			LM_ERR("routing_key size (%d) > max %d\n", routing_key_s.len, MAX_ROUTING_KEY_SIZE);
 			return -1;
 		}
 
-		json_object_put(j);
-
-		return kz_amqp_pipe_send(&exchange_s, &routing_key_s, &json_s );
-
-
+		return ki_kz_amqp_publish(msg, &exchange_s, &routing_key_s, &pl_s);
 };
 
+int ki_kz_amqp_publish(sip_msg_t* msg, str* exchange, str* routing_key, str* payload)
+{
+	  char *pl = ((str*)payload)->s;
+	  struct json_object *j = json_tokener_parse(pl);
+
+	  if (j==NULL) {
+	  	  LM_ERR("empty or invalid JSON payload : %.*s\n", payload->len, payload->s);
+	  	  return -1;
+	  }
+
+	  json_object_put(j);
+	  return kz_amqp_pipe_send(exchange, routing_key, payload);
+}
+
+int kz_amqp_publish(struct sip_msg* msg, char* exchange, char* routing_key, char* payload)
+{
+	return kz_amqp_publish_ex(msg, exchange, routing_key, payload, NULL);
+}
 
 char* last_payload_result = NULL;
 
@@ -1255,7 +1271,7 @@ int kz_pv_get_last_query_result(struct sip_msg *msg, pv_param_t *param,	pv_value
 	return last_payload_result == NULL ? pv_get_null(msg, param, res) : pv_get_strzval(msg, param, res, last_payload_result);
 }
 
-int kz_amqp_async_query(struct sip_msg* msg, char* _exchange, char* _routing_key, char* _payload, char* _cb_route, char* _err_route)
+int kz_amqp_async_query_ex(struct sip_msg* msg, char* _exchange, char* _routing_key, char* _payload, char* _cb_route, char* _err_route, char* _pub_flags)
 {
 	  str json_s;
 	  str exchange_s;
@@ -1291,9 +1307,14 @@ int kz_amqp_async_query(struct sip_msg* msg, char* _exchange, char* _routing_key
 		  goto error;
 	  }
 
+	  if (routing_key_s.len > MAX_ROUTING_KEY_SIZE) {
+		  LM_ERR("routing_key size (%d) > max %d\n", routing_key_s.len, MAX_ROUTING_KEY_SIZE);
+		  return -1;
+	  }
+
 	  json_obj = json_tokener_parse(json_s.s);
 
-	  if (is_error(json_obj)) {
+	  if (json_obj==NULL) {
 		  LM_ERR("empty or invalid JSON payload : %*.s\n", json_s.len, json_s.s);
 		  goto error;
 	  }
@@ -1395,6 +1416,11 @@ exit:
 	    return ret;
 };
 
+int kz_amqp_async_query(struct sip_msg* msg, char* _exchange, char* _routing_key, char* _payload, char* _cb_route, char* _err_route)
+{
+	return kz_amqp_async_query_ex(msg, _exchange, _routing_key, _payload, _cb_route, _err_route, NULL);
+}
+
 void kz_amqp_reset_last_result()
 {
 	if(last_payload_result)
@@ -1439,9 +1465,14 @@ int kz_amqp_query_ex(struct sip_msg* msg, char* exchange, char* routing_key, cha
 			return -1;
 		}
 
+		if (routing_key_s.len > MAX_ROUTING_KEY_SIZE) {
+			LM_ERR("routing_key size (%d) > max %d\n", routing_key_s.len, MAX_ROUTING_KEY_SIZE);
+			return -1;
+		}
+
 		struct json_object *j = json_tokener_parse(json_s.s);
 
-		if (is_error(j)) {
+		if (j==NULL) {
 			LM_ERR("empty or invalid JSON payload : %*.s\n", json_s.len, json_s.s);
 			return -1;
 		}
@@ -1515,7 +1546,7 @@ kz_amqp_queue_ptr kz_amqp_queue_new(str *name)
 {
 	kz_amqp_queue_ptr queue = (kz_amqp_queue_ptr) shm_malloc(sizeof(kz_amqp_queue));
 	if(queue == NULL) {
-		LM_ERR("NO MORE SHARED MEMORY!");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 	memset(queue, 0, sizeof(kz_amqp_queue));
@@ -1542,7 +1573,7 @@ kz_amqp_queue_ptr kz_amqp_queue_from_json(str *name, json_object* json_obj)
 	kz_amqp_queue_ptr queue = kz_amqp_queue_new(name);
 
 	if(queue == NULL) {
-		LM_ERR("NO MORE SHARED MEMORY!");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 
@@ -1585,7 +1616,7 @@ kz_amqp_exchange_ptr kz_amqp_exchange_new(str *name, str* type)
 {
 	kz_amqp_exchange_ptr exchange = (kz_amqp_exchange_ptr) shm_malloc(sizeof(kz_amqp_exchange));
 	if(exchange == NULL) {
-		LM_ERR("NO MORE SHARED MEMORY!");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 	memset(exchange, 0, sizeof(kz_amqp_exchange));
@@ -1630,7 +1661,7 @@ kz_amqp_exchange_ptr kz_amqp_exchange_from_json(str *name, json_object* json_obj
 
 	exchange = kz_amqp_exchange_new(name, &type);
 	if(exchange == NULL) {
-		LM_ERR("NO MORE SHARED MEMORY!");
+		SHM_MEM_ERROR;
 		return NULL;
 	}
 
@@ -1707,7 +1738,7 @@ kz_amqp_routings_ptr kz_amqp_routing_from_json(json_object* json_obj)
 		break;
 
    	default:
-   		LM_DBG("type not handled in routing");
+		LM_DBG("type not handled in routing\n");
    		break;
    	}
    	return ret;
@@ -1749,7 +1780,7 @@ kz_amqp_exchange_binding_ptr kz_amqp_exchange_binding_from_json(json_object* JOb
 		    	binding->from_exchange = exchange;
 		    	binding->routing = kz_amqp_routing_from_json(routingObj);
 		    	if(binding->routing == NULL) {
-		    		LM_DBG("invalid routing");
+					LM_DBG("invalid routing\n");
 		    		kz_amqp_exchange_bindings_free(binding);
 		    		binding = NULL;
 		    	} else {
@@ -1768,10 +1799,21 @@ kz_amqp_exchange_binding_ptr kz_amqp_exchange_binding_from_json(json_object* JOb
 }
 
 int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
+{
+	str payload_s = STR_NULL;
+
+	if (fixup_get_svalue(msg, (gparam_p)payload, &payload_s) != 0) {
+		LM_ERR("cannot get payload value\n");
+		return -1;
+	}
+
+	return ki_kz_amqp_subscribe(msg, &payload_s);
+}
+
+int ki_kz_amqp_subscribe(struct sip_msg* msg, str* payload)
 {
 	str exchange_s = STR_NULL;
 	str queue_s = STR_NULL;
-	str payload_s = STR_NULL;
 	str key_s = STR_NULL;
 	str subkey_s = STR_NULL;
 	int no_ack = 1;
@@ -1786,20 +1828,16 @@ int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
 	str* event_key = NULL;
     str* event_subkey = NULL;
 
-
-
 	json_obj_ptr json_obj = NULL;
 	struct json_object* tmpObj = NULL;
+	kz_amqp_bind_ptr bind = NULL;
+	kz_amqp_binding_ptr binding = NULL;
 
-	if (fixup_get_svalue(msg, (gparam_p)payload, &payload_s) != 0) {
-		LM_ERR("cannot get payload value\n");
-		return -1;
-	}
+	char* pl = payload->s;
+	json_obj = kz_json_parse(pl);
 
-	json_obj = kz_json_parse(payload_s.s);
 	if (json_obj == NULL)
 		return -1;
-    
 
 	json_extract_field("exchange", exchange_s);
 	json_extract_field("queue", queue_s);
@@ -1848,7 +1886,7 @@ int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
 	}
 
 	if(routing == NULL) {
-		LM_INFO("creating empty routing key : %s\n", payload_s.s);
+		LM_INFO("creating empty routing key : %s\n", pl);
 		routing = kz_amqp_routing_new("");
 	}
 
@@ -1864,7 +1902,7 @@ int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
 	}
 	queue = kz_amqp_queue_from_json(&queue_s, tmpObj);
 
-	kz_amqp_bind_ptr bind = kz_amqp_bind_alloc(exchange, exchange_binding, queue, routing, event_key, event_subkey);
+	bind = kz_amqp_bind_alloc(exchange, exchange_binding, queue, routing, event_key, event_subkey);
 	if(bind == NULL) {
 		LM_ERR("Could not allocate bind struct\n");
 		goto error;
@@ -1877,7 +1915,7 @@ int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
 	bind->consistent_worker_key = consistent_worker_key;
 
 
-	kz_amqp_binding_ptr binding = shm_malloc(sizeof(kz_amqp_binding));
+	binding = shm_malloc(sizeof(kz_amqp_binding));
 	if(binding == NULL) {
 		LM_ERR("Could not allocate binding struct\n");
 		goto error;
@@ -1919,7 +1957,8 @@ int kz_amqp_subscribe_simple(struct sip_msg* msg, char* exchange, char* exchange
 	kz_amqp_exchange_ptr exchange_ptr = NULL;
 	kz_amqp_queue_ptr queue_ptr = NULL;
 	kz_amqp_routings_ptr routing_ptr = NULL;
-
+	kz_amqp_bind_ptr bind = NULL;
+	kz_amqp_binding_ptr binding = NULL;
 
 	if (fixup_get_svalue(msg, (gparam_p)exchange, &exchange_s) != 0) {
 		LM_ERR("cannot get exchange string value\n");
@@ -1945,7 +1984,7 @@ int kz_amqp_subscribe_simple(struct sip_msg* msg, char* exchange, char* exchange
 	queue_ptr = kz_amqp_queue_new(&queue_s);
 	routing_ptr = kz_amqp_routing_new(routing_key_s.s);
 
-	kz_amqp_bind_ptr bind = kz_amqp_bind_alloc(exchange_ptr, NULL, queue_ptr, routing_ptr, NULL, NULL);
+	bind = kz_amqp_bind_alloc(exchange_ptr, NULL, queue_ptr, routing_ptr, NULL, NULL);
 	if(bind == NULL) {
 		LM_ERR("Could not allocate bind struct\n");
 		goto error;
@@ -1953,7 +1992,7 @@ int kz_amqp_subscribe_simple(struct sip_msg* msg, char* exchange, char* exchange
 
 	bind->no_ack = 1;
 
-	kz_amqp_binding_ptr binding = shm_malloc(sizeof(kz_amqp_binding));
+	binding = shm_malloc(sizeof(kz_amqp_binding));
 	if(binding == NULL) {
 		LM_ERR("Could not allocate binding struct\n");
 		goto error;
@@ -1992,33 +2031,33 @@ error:
 
 #define hexint(C) (C < 10?('0' + C):('A'+ C - 10))
 
-char *kz_amqp_util_encode(const str * key, char *dest) {
-    if ((key->len == 1) && (key->s[0] == '#' || key->s[0] == '*')) {
-	*dest++ = key->s[0];
-	return dest;
-    }
+void kz_amqp_util_encode(const str * key, char *pdest) {
     char *p, *end;
-    for (p = key->s, end = key->s + key->len; p < end; p++) {
-	if (KEY_SAFE(*p)) {
-	    *dest++ = *p;
-	} else if (*p == '.') {
-	    memcpy(dest, "\%2E", 3);
-	    dest += 3;
-	} else if (*p == ' ') {
-	    *dest++ = '+';
-	} else {
-	    *dest++ = '%';
-	    sprintf(dest, "%c%c", hexint(HI4(*p)), hexint(LO4(*p)));
-	    dest += 2;
-	}
+	char *dest = pdest;
+    if ((key->len == 1) && (key->s[0] == '#' || key->s[0] == '*')) {
+    	*dest++ = key->s[0];
+    	return;
+    }
+    for (p = key->s, end = key->s + key->len; p < end && ((dest - pdest) < MAX_ROUTING_KEY_SIZE); p++) {
+		if (KEY_SAFE(*p)) {
+			*dest++ = *p;
+		} else if (*p == '.') {
+			memcpy(dest, "\%2E", 3);
+			dest += 3;
+		} else if (*p == ' ') {
+			*dest++ = '+';
+		} else {
+			*dest++ = '%';
+			sprintf(dest, "%c%c", hexint(HI4(*p)), hexint(LO4(*p)));
+			dest += 2;
+		}
     }
     *dest = '\0';
-    return dest;
 }
 
 int kz_amqp_encode_ex(str* unencoded, pv_value_p dst_val)
 {
-	char routing_key_buff[256];
+	char routing_key_buff[MAX_ROUTING_KEY_SIZE+1];
 	memset(routing_key_buff,0, sizeof(routing_key_buff));
 	kz_amqp_util_encode(unencoded, routing_key_buff);
 
@@ -2045,6 +2084,11 @@ int kz_amqp_encode(struct sip_msg* msg, char* unencoded, char* encoded)
 		return -1;
 	}
 
+	if (unencoded_s.len > MAX_ROUTING_KEY_SIZE) {
+		LM_ERR("routing_key size (%d) > max %d\n", unencoded_s.len, MAX_ROUTING_KEY_SIZE);
+		return -1;
+	}
+
 	kz_amqp_encode_ex(&unencoded_s, &dst_val);
 	dst_pv->setf(msg, &dst_pv->pvp, (int)EQ_T, &dst_val);
 
@@ -2261,12 +2305,18 @@ int kz_amqp_send_receive(kz_amqp_server_ptr srv, kz_amqp_cmd_ptr cmd )
 }
 
 char* eventData = NULL;
+char* eventKey = NULL;
 
 int kz_pv_get_event_payload(struct sip_msg *msg, pv_param_t *param,	pv_value_t *res)
 {
 	return eventData == NULL ? pv_get_null(msg, param, res) : pv_get_strzval(msg, param, res, eventData);
 }
 
+int kz_pv_get_event_routing_key(struct sip_msg *msg, pv_param_t *param,	pv_value_t *res)
+{
+	return eventKey == NULL ? pv_get_null(msg, param, res) : pv_get_strzval(msg, param, res, eventKey);
+}
+
 int kz_amqp_consumer_fire_event(char *eventkey)
 {
 	sip_msg_t *fmsg;
@@ -2294,62 +2344,102 @@ int kz_amqp_consumer_fire_event(char *eventkey)
 	return 0;
 }
 
-void kz_amqp_consumer_event(char *payload, char* event_key, char* event_subkey)
+static void kz_amqp_consumer_event_cfg(kz_amqp_consumer_delivery_ptr Evt, json_obj_ptr json_obj)
 {
-    json_obj_ptr json_obj = NULL;
-    str ev_name = {0, 0}, ev_category = {0, 0};
-    char buffer[512];
-    char * p;
+	str ev_name = {0, 0}, ev_category = {0, 0};
+	char buffer[512];
+	char * p;
 
-    eventData = payload;
 
-    json_obj = kz_json_parse(payload);
-    if (json_obj == NULL)
-		return;
+	char* key = (Evt->event_key == NULL ? dbk_consumer_event_key.s : Evt->event_key);
+	char* subkey = (Evt->event_subkey == NULL ? dbk_consumer_event_subkey.s : Evt->event_subkey);
 
-    char* key = (event_key == NULL ? dbk_consumer_event_key.s : event_key);
-    char* subkey = (event_subkey == NULL ? dbk_consumer_event_subkey.s : event_subkey);
+	json_extract_field(key, ev_category);
+	if(ev_category.len == 0 && Evt->event_key) {
+		ev_category.s = Evt->event_key;
+		ev_category.len = strlen(Evt->event_key);
+	}
 
-    json_extract_field(key, ev_category);
-    if(ev_category.len == 0 && event_key) {
-	    ev_category.s = event_key;
-	    ev_category.len = strlen(event_key);
-    }
+	json_extract_field(subkey, ev_name);
+	if(ev_name.len == 0 && Evt->event_subkey) {
+		ev_name.s = Evt->event_subkey;
+		ev_name.len = strlen(Evt->event_subkey);
+	}
 
-    json_extract_field(subkey, ev_name);
-    if(ev_name.len == 0 && event_subkey) {
-	    ev_name.s = event_subkey;
-	    ev_name.len = strlen(event_subkey);
-    }
-
-    sprintf(buffer, "kazoo:consumer-event-%.*s-%.*s",ev_category.len, ev_category.s, ev_name.len, ev_name.s);
-    for (p=buffer ; *p; ++p) *p = tolower(*p);
-    for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
-    if(kz_amqp_consumer_fire_event(buffer) != 0) {
-        sprintf(buffer, "kazoo:consumer-event-%.*s",ev_category.len, ev_category.s);
-        for (p=buffer ; *p; ++p) *p = tolower(*p);
-        for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
-        if(kz_amqp_consumer_fire_event(buffer) != 0) {
-            sprintf(buffer, "kazoo:consumer-event-%s-%s", key, subkey);
-            for (p=buffer ; *p; ++p) *p = tolower(*p);
-            for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
-            if(kz_amqp_consumer_fire_event(buffer) != 0) {
-                sprintf(buffer, "kazoo:consumer-event-%s", key);
-                for (p=buffer ; *p; ++p) *p = tolower(*p);
-                for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
+	sprintf(buffer, "kazoo:consumer-event-%.*s-%.*s",ev_category.len, ev_category.s, ev_name.len, ev_name.s);
+	for (p=buffer ; *p; ++p) *p = tolower(*p);
+	for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
+	if(kz_amqp_consumer_fire_event(buffer) != 0) {
+		sprintf(buffer, "kazoo:consumer-event-%.*s",ev_category.len, ev_category.s);
+		for (p=buffer ; *p; ++p) *p = tolower(*p);
+		for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
+		if(kz_amqp_consumer_fire_event(buffer) != 0) {
+			sprintf(buffer, "kazoo:consumer-event-%s-%s", key, subkey);
+			for (p=buffer ; *p; ++p) *p = tolower(*p);
+			for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
+			if(kz_amqp_consumer_fire_event(buffer) != 0) {
+				sprintf(buffer, "kazoo:consumer-event-%s", key);
+				for (p=buffer ; *p; ++p) *p = tolower(*p);
+				for (p=buffer ; *p; ++p) if(*p == '_') *p = '-';
 				if(kz_amqp_consumer_fire_event(buffer) != 0) {
 					sprintf(buffer, "kazoo:consumer-event");
 					if(kz_amqp_consumer_fire_event(buffer) != 0) {
-						LM_ERR("kazoo:consumer-event not found");
+						LM_ERR("kazoo:consumer-event not found\n");
 					}
 				}
-            }
-        }
-    }
+			}
+		}
+	}
+}
+
+static void kz_amqp_consumer_event_kemi(void)
+{
+	sr_kemi_eng_t *keng = NULL;
+    int rtb;
+	
+	keng = sr_kemi_eng_get();
+	if(keng!=NULL) {
+		sip_msg_t *msg;
+		str evrtname = str_init("kazoo:consumer-event");
+
+		rtb = get_route_type();
+		msg = faked_msg_next();
+		if(sr_kemi_route(keng, msg, EVENT_ROUTE, &kazoo_event_callback, &evrtname)<0)                {
+			LM_ERR("error running event route kemi callback\n");
+		}
+
+		set_route_type(rtb);
+	} else {
+		LM_ERR("no event route or kemi callback found for execution\n");
+	}
+
+}
+
+void kz_amqp_consumer_event(kz_amqp_consumer_delivery_ptr Evt)
+{
+	json_obj_ptr json_obj = NULL;
+
+	eventData = Evt->payload;
+	if(Evt->routing_key) {
+		eventKey = Evt->routing_key->s;
+	}
+
+	json_obj = kz_json_parse(Evt->payload);
+	if (json_obj == NULL)
+		return;
+
+	if (kazoo_kemi_enabled) {
+		kz_amqp_consumer_event_kemi();
+	} 
+	else {
+		kz_amqp_consumer_event_cfg(Evt, json_obj);
+	}
+
 	if(json_obj)
-    	json_object_put(json_obj);
+		json_object_put(json_obj);
 
 	eventData = NULL;
+	eventKey = NULL;
 }
 
 int check_timeout(struct timeval *now, struct timeval *start, struct timeval *timeout)
@@ -2369,7 +2459,7 @@ void kz_amqp_send_consumer_event_ex(char* payload, char* event_key, char* event_
 {
 	kz_amqp_consumer_delivery_ptr ptr = (kz_amqp_consumer_delivery_ptr) shm_malloc(sizeof(kz_amqp_consumer_delivery));
 	if(ptr == NULL) {
-		LM_ERR("NO MORE SHARED MEMORY!");
+		SHM_MEM_ERROR;
 		return;
 	}
 	memset(ptr, 0, sizeof(kz_amqp_consumer_delivery));
@@ -2408,24 +2498,24 @@ void kz_amqp_fire_connection_event(char *event, char* host, char* zone)
 
 void kz_amqp_cb_ok(kz_amqp_cmd_ptr cmd)
 {
-	int n = route_lookup(&main_rt, cmd->cb_route);
+	int n = route_lookup(&onreply_rt, cmd->cb_route);
 	if(n==-1) {
 		/* route block not found in the configuration file */
 		return;
 	}
-	struct action *a = main_rt.rlist[n];
+	struct action *a = onreply_rt.rlist[n];
 	tmb.t_continue(cmd->t_hash, cmd->t_label, a);
 	ksr_msg_env_reset();
 }
 
 void kz_amqp_cb_error(kz_amqp_cmd_ptr cmd)
 {
-	int n = route_lookup(&main_rt, cmd->err_route);
+	int n = route_lookup(&failure_rt, cmd->err_route);
 	if(n==-1) {
 		/* route block not found in the configuration file */
 		return;
 	}
-	struct action *a = main_rt.rlist[n];
+	struct action *a = failure_rt.rlist[n];
 	tmb.t_continue(cmd->t_hash, cmd->t_label, a);
 	ksr_msg_env_reset();
 }
@@ -2435,7 +2525,7 @@ int kz_send_worker_error_event(kz_amqp_cmd_ptr cmd)
 	cmd->return_code = -1;
 	kz_amqp_consumer_delivery_ptr ptr = (kz_amqp_consumer_delivery_ptr) shm_malloc(sizeof(kz_amqp_consumer_delivery));
 	if(ptr == NULL) {
-		LM_ERR("NO MORE SHARED MEMORY!");
+		SHM_MEM_ERROR;
 		return 0;
 	}
 	memset(ptr, 0, sizeof(kz_amqp_consumer_delivery));
@@ -2626,12 +2716,13 @@ void kz_amqp_reconnect_cb(int fd, short event, void *arg)
 	LM_DBG("attempting to reconnect now.\n");
 	kz_amqp_conn_ptr connection = (kz_amqp_conn_ptr)arg;
 
+	kz_amqp_timer_destroy(&connection->reconnect);
+
 	if (connection->state == KZ_AMQP_CONNECTION_OPEN) {
 		LM_WARN("trying to connect an already connected server.\n");
 		return;
 	}
 
-	kz_amqp_timer_destroy(&connection->reconnect);
 	kz_amqp_connect(connection);
 }
 
@@ -2654,9 +2745,10 @@ int kz_amqp_publisher_send(kz_amqp_cmd_ptr cmd)
 	int sent = 0;
 	kz_amqp_zone_ptr g;
 	kz_amqp_server_ptr s;
+	kz_amqp_zone_ptr primary = kz_amqp_get_primary_zone();
 	for (g = kz_amqp_get_zones(); g != NULL && sent == 0; g = g->next) {
 		for (s = g->servers->head; s != NULL && sent == 0; s = s->next) {
-			if(cmd->server_id == s->id || cmd->server_id == 0) {
+			if(cmd->server_id == s->id || (cmd->server_id == 0 && g == primary)) {
 				if(s->producer->state == KZ_AMQP_CONNECTION_OPEN) {
 					if(cmd->type == KZ_AMQP_CMD_PUBLISH
 							|| cmd->type == KZ_AMQP_CMD_PUBLISH_BROADCAST
@@ -2818,9 +2910,13 @@ char* maybe_add_consumer_key(int server_id, amqp_bytes_t body)
     }
     char buffer[100];
     const char* server_id_str = json_object_get_string(server_id_obj);
-    sprintf(buffer, "consumer://%d/%s", server_id, server_id_str);
-    json_object_object_del(json_obj, BLF_JSON_SERVERID);
-    json_object_object_add(json_obj, BLF_JSON_SERVERID, json_object_new_string(buffer));
+    if(server_id_str && strlen(server_id_str) > 0) {
+    	sprintf(buffer, "consumer://%d/%s", server_id, server_id_str);
+        json_object_object_del(json_obj, BLF_JSON_SERVERID);
+    	json_object_object_add(json_obj, BLF_JSON_SERVERID, json_object_new_string(buffer));
+    } else {
+        json_object_object_del(json_obj, BLF_JSON_SERVERID);
+    }
     shm_free(payload);
     payload = kz_amqp_bytes_dup(amqp_cstring_bytes((char*)json_object_to_json_string(json_obj)));
    	json_object_put(json_obj);
@@ -2833,6 +2929,7 @@ void kz_send_targeted_cmd(int server_id, amqp_bytes_t body)
     char* server_id_str = NULL;
     kz_amqp_cmd_ptr cmd = NULL;
     json_object* JObj = NULL;
+    json_obj_ptr json_obj = NULL;
 	char* payload = kz_local_amqp_bytes_dup(body);
 
 	if(payload == NULL) {
@@ -2840,7 +2937,7 @@ void kz_send_targeted_cmd(int server_id, amqp_bytes_t body)
 		goto error;
 	}
 
-	json_obj_ptr json_obj = kz_json_parse(payload );
+	json_obj = kz_json_parse(payload );
     if (json_obj == NULL) {
 		LM_ERR("error parsing json payload\n");
 		goto error;
@@ -2864,9 +2961,13 @@ void kz_send_targeted_cmd(int server_id, amqp_bytes_t body)
     JObj = kz_json_get_object(json_obj, BLF_JSON_SERVERID);
     if(JObj != NULL) {
     	server_id_str = (char*) json_object_get_string(JObj);
-        sprintf(buffer, "consumer://%d/%s", server_id, server_id_str);
-        json_object_object_del(json_obj, BLF_JSON_SERVERID);
-        json_object_object_add(json_obj, BLF_JSON_SERVERID, json_object_new_string(buffer));
+        if(server_id_str && strlen(server_id_str) > 0) {
+        	sprintf(buffer, "consumer://%d/%s", server_id, server_id_str);
+            json_object_object_del(json_obj, BLF_JSON_SERVERID);
+        	json_object_object_add(json_obj, BLF_JSON_SERVERID, json_object_new_string(buffer));
+        } else {
+            json_object_object_del(json_obj, BLF_JSON_SERVERID);
+        }
     }
 
     cmd->return_payload = kz_amqp_string_dup((char*)json_object_to_json_string(json_obj));
@@ -2927,9 +3028,13 @@ void kz_amqp_send_worker_event(kz_amqp_server_ptr server_ptr, amqp_envelope_t* e
     JObj = kz_json_get_object(json_obj, BLF_JSON_SERVERID);
     if(JObj != NULL) {
         const char* _kz_server_id_str = json_object_get_string(JObj);
-        sprintf(buffer, "consumer://%d/%s", _kz_server_id, _kz_server_id_str);
-        json_object_object_del(json_obj, BLF_JSON_SERVERID);
-        json_object_object_add(json_obj, BLF_JSON_SERVERID, json_object_new_string(buffer));
+        if(_kz_server_id_str && strlen(_kz_server_id_str) > 0) {
+        	sprintf(buffer, "consumer://%d/%s", _kz_server_id, _kz_server_id_str);
+            json_object_object_del(json_obj, BLF_JSON_SERVERID);
+        	json_object_object_add(json_obj, BLF_JSON_SERVERID, json_object_new_string(buffer));
+        } else {
+            json_object_object_del(json_obj, BLF_JSON_SERVERID);
+        }
     }
 
     json_object_object_add(json_obj, BLF_JSON_BROKER_ZONE, json_object_new_string(server_ptr->zone->zone));
@@ -2950,7 +3055,7 @@ void kz_amqp_send_worker_event(kz_amqp_server_ptr server_ptr, amqp_envelope_t* e
 
 	ptr = (kz_amqp_consumer_delivery_ptr) shm_malloc(sizeof(kz_amqp_consumer_delivery));
 	if(ptr == NULL) {
-		LM_ERR("NO MORE SHARED MEMORY!");
+		SHM_MEM_ERROR;
 		goto error;
 	}
 	memset(ptr, 0, sizeof(kz_amqp_consumer_delivery));
@@ -3082,26 +3187,24 @@ int kz_amqp_consumer_proc(kz_amqp_server_ptr server_ptr)
 		LM_DBG("CONSUMER INIT DONE\n");
 
 		while(OK) {
-//			payload = NULL;
 			amqp_envelope_t envelope;
 			amqp_maybe_release_buffers(consumer->conn);
 			amqp_rpc_reply_t reply = amqp_consume_message(consumer->conn, &envelope, NULL, 0);
 			switch(reply.reply_type) {
 			case AMQP_RESPONSE_LIBRARY_EXCEPTION:
+				OK=0;
 				switch(reply.library_error) {
 				case AMQP_STATUS_HEARTBEAT_TIMEOUT:
 					LM_ERR("AMQP_STATUS_HEARTBEAT_TIMEOUT\n");
-					OK=0;
 					break;
 				case AMQP_STATUS_TIMEOUT:
 					break;
 				case AMQP_STATUS_UNEXPECTED_STATE:
 					LM_DBG("AMQP_STATUS_UNEXPECTED_STATE\n");
-					OK = kz_amqp_consume_error(consumer);
+					kz_amqp_consume_error(consumer);
 					break;
 				default:
 					LM_ERR("AMQP_RESPONSE_LIBRARY_EXCEPTION %i\n", reply.library_error);
-					OK = 0;
 					break;
 				};
 				break;
@@ -3121,6 +3224,7 @@ int kz_amqp_consumer_proc(kz_amqp_server_ptr server_ptr)
 					if(OK)
 						kz_amqp_send_worker_event(server_ptr, &envelope, consumer_channels[idx].consumer);
 				}
+				amqp_destroy_envelope(&envelope);
 				break;
 			case AMQP_RESPONSE_SERVER_EXCEPTION:
 				LM_ERR("AMQP_RESPONSE_SERVER_EXCEPTION in consume\n");
@@ -3132,7 +3236,6 @@ int kz_amqp_consumer_proc(kz_amqp_server_ptr server_ptr)
 				OK = 0;
 				break;
 			};
-			amqp_destroy_envelope(&envelope);
 		}
 
     	kz_amqp_connection_close(consumer);
@@ -3170,7 +3273,8 @@ void kz_amqp_consumer_worker_cb(int fd, short event, void *arg)
 			lock_release(&cmd->lock);
 		}
 	} else {
-		kz_amqp_consumer_event(Evt->payload, Evt->event_key, Evt->event_subkey);
+//		kz_amqp_consumer_event(Evt->payload, Evt->event_key, Evt->event_subkey);
+		kz_amqp_consumer_event(Evt);
 	}
 
 	kz_amqp_free_consumer_delivery(Evt);
@@ -3184,12 +3288,14 @@ int kz_amqp_consumer_worker_proc(int cmd_pipe)
 	set_non_blocking(cmd_pipe);
 	event_set(&pipe_ev, cmd_pipe, EV_READ | EV_PERSIST, kz_amqp_consumer_worker_cb, &pipe_ev);
 	event_add(&pipe_ev, NULL);
-	event_dispatch();
-	return 0;
+
+	return event_dispatch();
 }
 
 void kz_amqp_timer_destroy(kz_amqp_timer_ptr* pTimer)
 {
+	if(!pTimer)
+		return;
 	kz_amqp_timer_ptr timer = *pTimer;
 	if (timer->ev != NULL) {
 		event_del(timer->ev);
