@@ -31,12 +31,12 @@
 #include "../../core/hashes.h"
 #include "../../core/ut.h"
 #include "../../lib/srdb1/db_ut.h"
-#ifdef WITH_XAVP
 #include "../../core/xavp.h"
-#endif
 
 #include "sql_api.h"
 
+extern int sqlops_results_maxsize;
+
 sql_con_t *_sql_con_root = NULL;
 sql_result_t *_sql_result_root = NULL;
 
@@ -139,10 +139,11 @@ int pv_get_sqlrows(struct sip_msg *msg,  pv_param_t *param,
 	return pv_get_sintval(msg, param, res, con->dbf.affected_rows(con->dbh));
 }
 
-int sql_connect(void)
+int sql_connect(int mode)
 {
 	sql_con_t *sc;
 	sc = _sql_con_root;
+	LM_DBG("trying to connect to database with mode %d\n", mode);
 	while(sc)
 	{
 		if (db_bind_mod(&sc->db_url, &sc->dbf))
@@ -160,15 +161,39 @@ int sql_connect(void)
 		sc->dbh = sc->dbf.init(&sc->db_url);
 		if (sc->dbh==NULL)
 		{
-			LM_ERR("failed to connect to the database [%.*s]\n",
-					sc->name.len, sc->name.s);
-			return -1;
+			if(!mode) {
+				LM_ERR("failed to connect to the database [%.*s]\n",
+						sc->name.len, sc->name.s);
+				return -1;
+			} else {
+				LM_INFO("failed to connect to the database [%.*s] - trying next\n",
+						sc->name.len, sc->name.s);
+			}
 		}
 		sc = sc->next;
 	}
 	return 0;
 }
 
+int sql_reconnect(sql_con_t *sc)
+{
+	if(sc==NULL) {
+		LM_ERR("connection structure not initialized\n");
+		return -1;
+	}
+	if (sc->dbh!=NULL) {
+		/* already connected */
+		return 0;
+	}
+	sc->dbh = sc->dbf.init(&sc->db_url);
+	if (sc->dbh==NULL) {
+		LM_ERR("failed to connect to the database [%.*s]\n",
+					sc->name.len, sc->name.s);
+		return -1;
+	}
+	return 0;
+}
+
 void sql_disconnect(void)
 {
 	sql_con_t *sc;
@@ -186,17 +211,25 @@ sql_result_t* sql_get_result(str *name)
 {
 	sql_result_t *sr;
 	unsigned int resid;
+	int i;
 
 	resid = core_case_hash(name, 0, 0);
 
 	sr = _sql_result_root;
+	i = 0;
 	while(sr)
 	{
 		if(sr->resid==resid && sr->name.len==name->len
 				&& strncmp(sr->name.s, name->s, name->len)==0)
 			return sr;
+		i++;
 		sr = sr->next;
 	}
+	if(i>sqlops_results_maxsize)
+	{
+		LM_ERR("too many result containers defined\n");
+		return NULL;
+	}
 	sr = (sql_result_t*)pkg_malloc(sizeof(sql_result_t) + name->len);
 	if(sr==NULL)
 	{
@@ -262,7 +295,7 @@ int sql_do_query(sql_con_t *con, str *query, sql_result_t *res)
 	if(con->dbf.raw_query(con->dbh, query, &db_res)!=0)
 	{
 		LM_ERR("cannot do the query [%.*s]\n",
-				(query->len>32)?32:query->len, query->s);
+				(query->len>64)?64:query->len, query->s);
 		return -1;
 	}
 
@@ -394,14 +427,15 @@ int sql_do_query(sql_con_t *con, str *query, sql_result_t *res)
 					res->vals[i][j].value.s.len = 0;
 					continue;
 				}
-				res->vals[i][j].value.s.s 
-					= (char*)pkg_malloc(sv.len*sizeof(char));
+				res->vals[i][j].value.s.s
+					= (char*)pkg_malloc((sv.len+1)*sizeof(char));
 				if(res->vals[i][j].value.s.s==NULL)
 				{
 					LM_ERR("no more memory\n");
 					goto error;
 				}
 				memcpy(res->vals[i][j].value.s.s, sv.s, sv.len);
+				res->vals[i][j].value.s.s[sv.len] = '\0';
 				res->vals[i][j].value.s.len = sv.len;
 			}
 		}
@@ -435,7 +469,6 @@ int sql_do_query_async(sql_con_t *con, str *query)
 	return 1;
 }
 
-#ifdef WITH_XAVP
 int sql_exec_xquery(struct sip_msg *msg, sql_con_t *con, str *query,
 		str *xavp)
 {
@@ -534,6 +567,7 @@ int sql_exec_xquery(struct sip_msg *msg, sql_con_t *con, str *query,
 	return 1;
 }
 
+
 int sql_do_xquery(struct sip_msg *msg, sql_con_t *con, pv_elem_t *query,
 		pv_elem_t *res)
 {
@@ -557,8 +591,6 @@ int sql_do_xquery(struct sip_msg *msg, sql_con_t *con, pv_elem_t *query,
 	return sql_exec_xquery(msg, con, &sv, &xavp);
 }
 
-#endif
-
 
 int sql_do_pvquery(struct sip_msg *msg, sql_con_t *con, pv_elem_t *query,
 		pvname_list_t *res)
@@ -696,17 +728,27 @@ int sqlops_do_query(str *scon, str *squery, str *sres)
 	sql_con_t *con = NULL;
 	sql_result_t *res = NULL;
 
+	if (scon == NULL || scon->s == NULL)
+	{
+		LM_ERR("invalid connection name\n");
+		goto error;
+	}
+
 	con = sql_get_connection(scon);
 	if(con==NULL)
 	{
 		LM_ERR("invalid connection [%.*s]\n", scon->len, scon->s);
 		goto error;
 	}
-	res = sql_get_result(sres);
-	if(res==NULL)
+	/* Result parameter is optional */
+	if (sres && sres->s)
 	{
-		LM_ERR("invalid result [%.*s]\n", sres->len, sres->s);
-		goto error;
+		res = sql_get_result(sres);
+		if(res==NULL)
+		{
+			LM_ERR("invalid result container [%.*s]\n", sres->len, sres->s);
+			goto error;
+		}
 	}
 	if(sql_do_query(con, squery, res)<0)
 		goto error;
@@ -723,10 +765,16 @@ int sqlops_get_value(str *sres, int i, int j, sql_val_t **val)
 {
 	sql_result_t *res = NULL;
 
+	if (sres == NULL || sres->s == NULL)
+	{
+		LM_ERR("invalid result name\n");
+		goto error;
+	}
+
 	res = sql_get_result(sres);
 	if(res==NULL)
 	{
-		LM_ERR("invalid result [%.*s]\n", sres->len, sres->s);
+		LM_ERR("invalid result container [%.*s]\n", sres->len, sres->s);
 		goto error;
 	}
 	if(i>=res->nrows)
@@ -753,10 +801,16 @@ int sqlops_is_null(str *sres, int i, int j)
 {
 	sql_result_t *res = NULL;
 
+	if (sres == NULL || sres->s == NULL)
+	{
+		LM_ERR("invalid result name\n");
+		goto error;
+	}
+
 	res = sql_get_result(sres);
 	if(res==NULL)
 	{
-		LM_ERR("invalid result [%.*s]\n", sres->len, sres->s);
+		LM_ERR("invalid result container [%.*s]\n", sres->len, sres->s);
 		goto error;
 	}
 	if(i>=res->nrows)
@@ -783,10 +837,16 @@ int sqlops_get_column(str *sres, int i, str *col)
 {
 	sql_result_t *res = NULL;
 
+	if (sres == NULL || sres->s == NULL)
+	{
+		LM_ERR("invalid result name\n");
+		goto error;
+	}
+
 	res = sql_get_result(sres);
 	if(res==NULL)
 	{
-		LM_ERR("invalid result [%.*s]\n", sres->len, sres->s);
+		LM_ERR("invalid result container [%.*s]\n", sres->len, sres->s);
 		goto error;
 	}
 	if(i>=res->ncols)
@@ -807,10 +867,16 @@ int sqlops_num_columns(str *sres)
 {
 	sql_result_t *res = NULL;
 
+	if (sres == NULL || sres->s == NULL)
+	{
+		LM_ERR("invalid result name\n");
+		goto error;
+	}
+
 	res = sql_get_result(sres);
 	if(res==NULL)
 	{
-		LM_ERR("invalid result [%.*s]\n", sres->len, sres->s);
+		LM_ERR("invalid result container [%.*s]\n", sres->len, sres->s);
 		goto error;
 	}
 	return res->ncols;
@@ -825,10 +891,16 @@ int sqlops_num_rows(str *sres)
 {
 	sql_result_t *res = NULL;
 
+	if (sres == NULL || sres->s == NULL)
+	{
+		LM_ERR("invalid result name\n");
+		goto error;
+	}
+
 	res = sql_get_result(sres);
 	if(res==NULL)
 	{
-		LM_ERR("invalid result [%.*s]\n", sres->len, sres->s);
+		LM_ERR("invalid result container [%.*s]\n", sres->len, sres->s);
 		goto error;
 	}
 	return res->nrows;
@@ -843,10 +915,16 @@ void sqlops_reset_result(str *sres)
 {
 	sql_result_t *res = NULL;
 
+	if (sres == NULL || sres->s == NULL)
+	{
+		LM_ERR("invalid result name\n");
+		return;
+	}
+
 	res = sql_get_result(sres);
 	if(res==NULL)
 	{
-		LM_ERR("invalid result [%.*s]\n", sres->len, sres->s);
+		LM_ERR("invalid result container [%.*s]\n", sres->len, sres->s);
 		return;
 	}
 	sql_reset_result(res);
@@ -861,6 +939,12 @@ int sqlops_do_xquery(sip_msg_t *msg, str *scon, str *squery, str *xavp)
 {
 	sql_con_t *con = NULL;
 
+	if (scon == NULL || scon->s == NULL)
+	{
+		LM_ERR("invalid connection name\n");
+		goto error;
+	}
+
 	con = sql_get_connection(scon);
 	if(con==NULL)
 	{
