@@ -19,7 +19,7 @@
 #include "../../core/ip_addr.h"
 #include "../../core/dprint.h"
 #include "../../core/dset.h"
-#include "../../lib/srutils/sruid.h"
+#include "../../core/utils/sruid.h"
 
 #include "ul_rpc.h"
 #include "dlist.h"
@@ -31,11 +31,11 @@
 /*! CSEQ nr used */
 #define RPC_UL_CSEQ 1
 /*! call-id used for ul_add and ul_rm_contact */
-static str rpc_ul_cid = str_init("dfjrewr12386fd6-343@kamailio.mi");
+static str rpc_ul_cid = str_init("dfjrewr12386fd6-343@kamailio.rpc");
 /*! path used for ul_add and ul_rm_contact */
 static str rpc_ul_path = str_init("dummypath");
 /*! user agent used for ul_add */
-static str rpc_ul_ua  = str_init("SIP Router MI Server");
+static str rpc_ul_ua  = str_init(NAME " SIP Router - RPC Server");
 
 extern sruid_t _ul_sruid;
 
@@ -65,10 +65,10 @@ int rpc_dump_contact(rpc_t* rpc, void* ctx, void *ih, ucontact_t* c)
 		return -1;
 	}
 	if (c->expires == 0) { if(rpc->struct_add(vh, "s", "Expires", "permanent")<0)
-	{
+		{
 			rpc->fault(ctx, 500, "Internal error adding expire");
 			return -1;
-	}
+		}
 	} else if (c->expires == UL_EXPIRED_TIME) {
 		if(rpc->struct_add(vh, "s", "Expires", "deleted")<0)
 		{
@@ -122,19 +122,19 @@ int rpc_dump_contact(rpc_t* rpc, void* ctx, void *ih, ucontact_t* c)
 		return -1;
 	}
 	if(rpc->struct_add(vh, "S", "User-Agent",
-			(c->user_agent.len)?&c->user_agent: &empty_str)<0)
+				(c->user_agent.len)?&c->user_agent: &empty_str)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding user-agent");
 		return -1;
 	}
 	if(rpc->struct_add(vh, "S", "Received",
-			(c->received.len)?&c->received: &empty_str)<0)
+				(c->received.len)?&c->received: &empty_str)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding received");
 		return -1;
 	}
 	if(rpc->struct_add(vh, "S", "Path",
-			(c->path.len)?&c->path: &empty_str)<0)
+				(c->path.len)?&c->path: &empty_str)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding path");
 		return -1;
@@ -144,12 +144,12 @@ int rpc_dump_contact(rpc_t* rpc, void* ctx, void *ih, ucontact_t* c)
 		rpc->fault(ctx, 500, "Internal error adding state");
 		return -1;
 	}
-	if(rpc->struct_add(vh, "d", "Flags", c->flags)<0)
+	if(rpc->struct_add(vh, "u", "Flags", c->flags)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding flags");
 		return -1;
 	}
-	if(rpc->struct_add(vh, "d", "CFlags", c->cflags)<0)
+	if(rpc->struct_add(vh, "u", "CFlags", c->cflags)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding cflags");
 		return -1;
@@ -159,7 +159,7 @@ int rpc_dump_contact(rpc_t* rpc, void* ctx, void *ih, ucontact_t* c)
 		rpc->fault(ctx, 500, "Internal error adding socket");
 		return -1;
 	}
-	if(rpc->struct_add(vh, "d", "Methods", c->methods)<0)
+	if(rpc->struct_add(vh, "u", "Methods", c->methods)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding methods");
 		return -1;
@@ -170,24 +170,44 @@ int rpc_dump_contact(rpc_t* rpc, void* ctx, void *ih, ucontact_t* c)
 		return -1;
 	}
 	if(rpc->struct_add(vh, "S", "Instance",
-			(c->instance.len)?&c->instance: &empty_str)<0)
+				(c->instance.len)?&c->instance: &empty_str)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding instance");
 		return -1;
 	}
-	if(rpc->struct_add(vh, "d", "Reg-Id", c->reg_id)<0)
+	if(rpc->struct_add(vh, "u", "Reg-Id", c->reg_id)<0)
 	{
 		rpc->fault(ctx, 500, "Internal error adding reg_id");
 		return -1;
 	}
+	if(rpc->struct_add(vh, "d", "Server-Id", c->server_id)<0)
+	{
+		rpc->fault(ctx, 500, "Internal error adding server_id");
+		return -1;
+	}
+	if(rpc->struct_add(vh, "d", "Tcpconn-Id", c->tcpconn_id)<0)
+	{
+		rpc->fault(ctx, 500, "Internal error adding tcpconn_id");
+		return -1;
+	}
+	if(rpc->struct_add(vh, "d", "Keepalive", c->keepalive)<0)
+	{
+		rpc->fault(ctx, 500, "Internal error adding keepalive");
+		return -1;
+	}
 	if(rpc->struct_add(vh, "d", "Last-Keepalive", (int)c->last_keepalive)<0)
 	{
-		rpc->fault(ctx, 500, "Internal error adding reg_id");
+		rpc->fault(ctx, 500, "Internal error adding last_keepalive");
+		return -1;
+	}
+	if(rpc->struct_add(vh, "d", "KA-Roundtrip", (int)c->ka_roundtrip)<0)
+	{
+		rpc->fault(ctx, 500, "Internal error adding keepalive roundtrip");
 		return -1;
 	}
 	if(rpc->struct_add(vh, "d", "Last-Modified", (int)c->last_modified)<0)
 	{
-		rpc->fault(ctx, 500, "Internal error adding reg_id");
+		rpc->fault(ctx, 500, "Internal error adding last_modified");
 		return -1;
 	}
 	return 0;
@@ -202,6 +222,8 @@ static void ul_rpc_dump(rpc_t* rpc, void* ctx)
 	int summary = 0;
 	ucontact_t* c;
 	void* th;
+	void* dah;
+	void* dh;
 	void* ah;
 	void* bh;
 	void* ih;
@@ -212,15 +234,28 @@ static void ul_rpc_dump(rpc_t* rpc, void* ctx)
 
 	if(brief.len==5 && (strncmp(brief.s, "brief", 5)==0))
 		summary = 1;
+
+	if (rpc->add(ctx, "{", &th) < 0)
+	{
+		rpc->fault(ctx, 500, "Internal error creating top rpc");
+		return;
+	}
+	if (rpc->struct_add(th, "[", "Domains", &dah) < 0)
+	{
+		rpc->fault(ctx, 500, "Internal error creating inner struct");
+		return;
+	}
 	
-	for( dl=root ; dl ; dl=dl->next ) {
+	for( dl=_ksr_ul_root ; dl ; dl=dl->next ) {
 		dom = dl->d;
-		if (rpc->add(ctx, "{", &th) < 0)
+
+		if (rpc->struct_add(dah, "{", "Domain", &dh) < 0)
 		{
-			rpc->fault(ctx, 500, "Internal error creating top rpc");
+			rpc->fault(ctx, 500, "Internal error creating inner struct");
 			return;
 		}
-		if(rpc->struct_add(th, "Sd[",
+
+		if(rpc->struct_add(dh, "Sd[",
 					"Domain",  &dl->name,
 					"Size",    (int)dom->size,
 					"AoRs",    &ah)<0)
@@ -237,23 +272,24 @@ static void ul_rpc_dump(rpc_t* rpc, void* ctx)
 				if(summary==1)
 				{
 					if(rpc->struct_add(ah, "S",
-							"AoR", &r->aor)<0)
-					{
-						rpc->fault(ctx, 500, "Internal error creating aor struct");
-						return;
-					}
-				} else {
-					if(rpc->struct_add(ah, "{",
-							"Info", &bh)<0)
+								"AoR", &r->aor)<0)
 					{
 						unlock_ulslot( dom, i);
 						rpc->fault(ctx, 500, "Internal error creating aor struct");
 						return;
 					}
-					if(rpc->struct_add(bh, "Sd[",
-							"AoR", &r->aor,
-							"HashID", r->aorhash,
-							"Contacts", &ih)<0)
+				} else {
+					if(rpc->struct_add(ah, "{",
+								"Info", &bh)<0)
+					{
+						unlock_ulslot( dom, i);
+						rpc->fault(ctx, 500, "Internal error creating aor struct");
+						return;
+					}
+					if(rpc->struct_add(bh, "Su[",
+								"AoR", &r->aor,
+								"HashID", r->aorhash,
+								"Contacts", &ih)<0)
 					{
 						unlock_ulslot( dom, i);
 						rpc->fault(ctx, 500, "Internal error creating aor struct");
@@ -273,14 +309,14 @@ static void ul_rpc_dump(rpc_t* rpc, void* ctx)
 		}
 
 		/* extra attributes node */
-		if(rpc->struct_add(th, "{", "Stats",    &sh)<0)
+		if(rpc->struct_add(dh, "{", "Stats",    &sh)<0)
 		{
 			rpc->fault(ctx, 500, "Internal error creating stats struct");
 			return;
 		}
 		if(rpc->struct_add(sh, "dd",
-				"Records", n,
-				"Max-Slots", max)<0)
+					"Records", n,
+					"Max-Slots", max)<0)
 		{
 			rpc->fault(ctx, 500, "Internal error adding stats");
 			return;
@@ -302,9 +338,9 @@ static inline udomain_t* rpc_find_domain(str* table)
 {
 	dlist_t* dom;
 
-	for( dom=root ; dom ; dom=dom->next ) {
+	for( dom=_ksr_ul_root ; dom ; dom=dom->next ) {
 		if ((dom->name.len == table->len) &&
-		!memcmp(dom->name.s, table->s, table->len))
+				!memcmp(dom->name.s, table->s, table->len))
 			return dom->d;
 	}
 	return 0;
@@ -323,7 +359,7 @@ static inline int rpc_fix_aor(str *aor)
 	char *p;
 
 	p = memchr( aor->s, '@', aor->len);
-	if (use_domain) {
+	if (ul_use_domain) {
 		if (p==NULL)
 			return -1;
 	} else {
@@ -366,7 +402,7 @@ static void ul_rpc_lookup(rpc_t* rpc, void* ctx)
 	/* look for table */
 	dom = rpc_find_domain( &table );
 	if (dom == NULL) {
-		rpc->fault(ctx, 500, "Domain not found");
+		rpc->fault(ctx, 500, "Domain table not found");
 		return;
 	}
 
@@ -385,19 +421,21 @@ static void ul_rpc_lookup(rpc_t* rpc, void* ctx)
 		return;
 	}
 
-	get_act_time();
+	ul_get_act_time();
 	rpl_tree = 0;
 
 	if (rpc->add(ctx, "{", &th) < 0)
 	{
+		release_urecord(rec);
 		unlock_udomain(dom, &aor);
 		rpc->fault(ctx, 500, "Internal error creating outer rpc");
 		return;
 	}
 	if(rpc->struct_add(th, "S[",
-			"AoR", &aor,
-			"Contacts", &ih)<0)
+				"AoR", &aor,
+				"Contacts", &ih)<0)
 	{
+		release_urecord(rec);
 		unlock_udomain(dom, &aor);
 		rpc->fault(ctx, 500, "Internal error creating aor struct");
 		return;
@@ -405,15 +443,16 @@ static void ul_rpc_lookup(rpc_t* rpc, void* ctx)
 
 	/* We have contacts, list them */
 	for( con=rec->contacts ; con ; con=con->next) {
-		if (VALID_CONTACT( con, act_time)) {
+		if (VALID_CONTACT(con, ul_act_time)) {
 			rpl_tree++;
 			if (rpc_dump_contact(rpc, ctx, ih, con) == -1) {
+				release_urecord(rec);
 				unlock_udomain(dom, &aor);
 				return;
 			}
 		}
 	}
-
+	release_urecord(rec);
 	unlock_udomain( dom, &aor);
 
 	if (rpl_tree==0) {
@@ -437,7 +476,7 @@ static void ul_rpc_rm_aor(rpc_t* rpc, void* ctx)
 	/* look for table */
 	dom = rpc_find_domain( &table );
 	if (dom == NULL) {
-		rpc->fault(ctx, 500, "Domain not found");
+		rpc->fault(ctx, 500, "Domain table not found");
 		return;
 	}
 
@@ -481,7 +520,7 @@ static void ul_rpc_rm_contact(rpc_t* rpc, void* ctx)
 	/* look for table */
 	dom = rpc_find_domain( &table );
 	if (dom == NULL) {
-		rpc->fault(ctx, 500, "Domain not found");
+		rpc->fault(ctx, 500, "Domain table not found");
 		return;
 	}
 
@@ -502,17 +541,20 @@ static void ul_rpc_rm_contact(rpc_t* rpc, void* ctx)
 
 	ret = get_ucontact( rec, &contact, &rpc_ul_cid, &rpc_ul_path, RPC_UL_CSEQ+1, &con);
 	if (ret < 0) {
+		release_urecord(rec);
 		unlock_udomain( dom, &aor);
 		rpc->fault(ctx, 500, "Internal error (can't get contact)");
 		return;
 	}
 	if (ret > 0) {
+		release_urecord(rec);
 		unlock_udomain( dom, &aor);
 		rpc->fault(ctx, 404, "Contact not found");
 		return;
 	}
 
 	if (delete_ucontact(rec, con) < 0) {
+		release_urecord(rec);
 		unlock_udomain( dom, &aor);
 		rpc->fault(ctx, 500, "Internal error (can't delete contact)");
 		return;
@@ -530,7 +572,9 @@ static const char* ul_rpc_rm_contact_doc[2] = {
 
 static void ul_rpc_flush(rpc_t* rpc, void* ctx)
 {
-	synchronize_all_udomains(0, 1);
+	if(synchronize_all_udomains(0, 1) != 0) {
+		rpc->fault(ctx, 500, "Operation failed");
+	}
 	return;
 }
 
@@ -539,6 +583,23 @@ static const char* ul_rpc_flush_doc[2] = {
 	0
 };
 
+/**
+ * test if a rpc parameter is set
+ * - used for optional parameters which can be given '.' or '0' value
+ *   in order to indicate is not intended to be set
+ * - return: 1 if parameter is set; 0 if parameter is not set
+ */
+int ul_rpc_is_param_set(str *p)
+{
+	if(p==NULL || p->len==0 || p->s==NULL)
+		return 0;
+	if(p->len>1)
+		return 1;
+	if((strncmp(p->s, ".", 1)==0) || (strncmp(p->s, "0", 1)==0))
+		return 0;
+	return 1;
+}
+
 /*!
  * \brief Add a new contact for an address of record
  * \note Expects 9 parameters: table name, AOR, contact, expires, Q,
@@ -550,6 +611,8 @@ static void ul_rpc_add(rpc_t* rpc, void* ctx)
 	str aor = {0, 0};
 	str contact = {0, 0};
 	str path = {0, 0};
+	str received = {0, 0};
+	str socket = {0, 0};
 	str temp = {0, 0};
 	double dtemp;
 	ucontact_info_t ci;
@@ -558,95 +621,114 @@ static void ul_rpc_add(rpc_t* rpc, void* ctx)
 	udomain_t *dom;
 	int ret;
 
-	memset( &ci, 0, sizeof(ucontact_info_t));
+	memset(&ci, 0, sizeof(ucontact_info_t));
 
-	ret = rpc->scan(ctx, "SSSdfSddd", &table, &aor, &contact, &ci.expires,
-		&dtemp, &path, &ci.flags, &ci.cflags, &ci.methods);
-	if(path.len==1 && (strncmp(path.s, "0", 1)==0))	{
-		LM_DBG("path == 0 -> unset\n");
-	}
-	else {
-		ci.path = &path;
-	}
-	LM_DBG("ret: %d table:%.*s aor:%.*s contact:%.*s expires:%d dtemp:%f path:%.*s flags:%d bflags:%d methods:%d\n",
-		ret, table.len, table.s, aor.len, aor.s, contact.len, contact.s,
-		(int) ci.expires, dtemp, (ci.path)?ci.path->len:0, (ci.path && ci.path->s)?ci.path->s:"", ci.flags, ci.cflags, (int) ci.methods);
-	if ( ret != 9) {
+	ret = rpc->scan(ctx, "SSSdfSuuu*SS", &table, &aor, &contact, &ci.expires,
+			&dtemp, &path, &ci.flags, &ci.cflags, &ci.methods, &received,
+			&socket);
+	if (ret < 9) {
+		LM_ERR("not enough parameters - read so far: %d\n", ret);
 		rpc->fault(ctx, 500, "Not enough parameters or wrong format");
 		return;
 	}
+	if(ul_rpc_is_param_set(&path)) {
+		ci.path = &path;
+	} else {
+		LM_DBG("path == 0 -> unset\n");
+	}
+	if(ret>9) {
+		/* received parameter */
+		if(ul_rpc_is_param_set(&received)) {
+			ci.received.s = received.s;
+			ci.received.len = received.len;
+		}
+	}
+	if(ret>10) {
+		/* socket parameter */
+		if(ul_rpc_is_param_set(&socket)) {
+			ci.sock = lookup_local_socket(&socket);
+		}
+	}
+	LM_DBG("ret: %d table:%.*s aor:%.*s contact:%.*s expires:%d"
+			" dtemp:%f path:%.*s flags:%d bflags:%d methods:%d\n",
+			ret, table.len, table.s, aor.len, aor.s, contact.len, contact.s,
+			(int) ci.expires, dtemp, (ci.path)?ci.path->len:0,
+			(ci.path && ci.path->s)?ci.path->s:"", ci.flags, ci.cflags,
+			(int) ci.methods);
 	ci.q = double2q(dtemp);
 	temp.s = q2str(ci.q, (unsigned int*)&temp.len);
 	LM_DBG("q:%.*s\n", temp.len, temp.s);
 	/* look for table */
 	dom = rpc_find_domain( &table );
 	if (dom == NULL) {
-		rpc->fault(ctx, 500, "Domain not found");
+		rpc->fault(ctx, 500, "Domain table not found");
 		return;
 	}
 
 	/* process the aor */
-	if ( rpc_fix_aor(&aor) != 0 ) {
+	if (rpc_fix_aor(&aor) != 0 ) {
 		rpc->fault(ctx, 500, "Domain missing in AOR");
 		return;
 	}
 
-	if(sruid_next(&_ul_sruid)<0)
+	if(sruid_next_safe(&_ul_sruid)<0)
 	{
 		rpc->fault(ctx, 500, "Can't obtain next uid");
 		return;
 	}
 	ci.ruid = _ul_sruid.uid;
+	ci.server_id = server_id;
 
-	lock_udomain( dom, &aor);
+	lock_udomain(dom, &aor);
 
-	ret = get_urecord( dom, &aor, &r);
+	ret = get_urecord(dom, &aor, &r);
 	if(ret==1) {
-		if (insert_urecord( dom, &aor, &r) < 0)
+		if (insert_urecord(dom, &aor, &r) < 0)
 		{
-			unlock_udomain( dom, &aor);
+			unlock_udomain(dom, &aor);
 			rpc->fault(ctx, 500, "Can't insert record");
 			return;
 		}
 		c = 0;
 	} else {
-		if (get_ucontact( r, &contact, &rpc_ul_cid, &rpc_ul_path, RPC_UL_CSEQ+1, &c) < 0)
+		if (get_ucontact( r, &contact, &rpc_ul_cid, &rpc_ul_path,
+					RPC_UL_CSEQ+1, &c) < 0)
 		{
-			unlock_udomain( dom, &aor);
+			unlock_udomain(dom, &aor);
 			rpc->fault(ctx, 500, "Can't get record");
 			return;
 		}
 	}
 
-	get_act_time();
+	ul_get_act_time();
 
 	ci.callid = &rpc_ul_cid;
 	ci.user_agent = &rpc_ul_ua;
 	ci.cseq = RPC_UL_CSEQ;
 	/* 0 expires means permanent contact */
 	if (ci.expires!=0)
-		ci.expires += act_time;
+		ci.expires += ul_act_time;
 
 	if (c) {
 		if (update_ucontact( r, c, &ci) < 0)
 		{
 			release_urecord(r);
-			unlock_udomain( dom, &aor);
+			unlock_udomain(dom, &aor);
 			rpc->fault(ctx, 500, "Can't update contact");
 			return;
 		}
 	} else {
-		if ( insert_ucontact( r, &contact, &ci, &c) < 0 )
+		if ( insert_ucontact(r, &contact, &ci, &c) < 0 )
 		{
 			release_urecord(r);
-			unlock_udomain( dom, &aor);
+			unlock_udomain(dom, &aor);
 			rpc->fault(ctx, 500, "Can't insert contact");
 			return;
 		}
 	}
 
 	release_urecord(r);
-	unlock_udomain( dom, &aor);
+	unlock_udomain(dom, &aor);
 	return;
 }
 
@@ -659,52 +741,53 @@ static const char* ul_rpc_add_doc[2] = {
 
 static void ul_rpc_db_users(rpc_t* rpc, void* ctx)
 {
-    str table = {0, 0};
-    char query[QUERY_LEN];
-    str query_str;
-    db1_res_t* res;
-    int count;
+	str table = {0, 0};
+	char query[QUERY_LEN];
+	str query_str;
+	db1_res_t* res = NULL;
+	int count = 0;
 
-    if (db_mode == NO_DB) {
-	rpc->fault(ctx, 500, "Command is not supported in db_mode=0");
-	return;
-    }
+	if (ul_db_mode == NO_DB) {
+		rpc->fault(ctx, 500, "Command is not supported in db_mode=0");
+		return;
+	}
 
-    if (rpc->scan(ctx, "S", &table) != 1) {
-	rpc->fault(ctx, 500, "Not enough parameters (table to lookup)");
-	return;
-    }
+	if (rpc->scan(ctx, "S", &table) != 1) {
+		rpc->fault(ctx, 500, "Not enough parameters (table to lookup)");
+		return;
+	}
 
-    if (user_col.len + domain_col.len + table.len + 32 > QUERY_LEN) {
-	rpc->fault(ctx, 500, "Too long database query");
-	return;
-    }
+	if (ul_user_col.len + ul_domain_col.len + table.len + 32 > QUERY_LEN) {
+		rpc->fault(ctx, 500, "Too long database query");
+		return;
+	}
 
-    if (!DB_CAPABILITY(ul_dbf, DB_CAP_RAW_QUERY)) {
-	rpc->fault(ctx, 500, "Database does not support raw queries");
-	return;
-    }
-    if (ul_dbf.use_table(ul_dbh, &table) < 0) {
-	rpc->fault(ctx, 500, "Failed to use table");
-	return;
-    }
-	
-    memset(query, 0, QUERY_LEN);
-    query_str.len = snprintf(query, QUERY_LEN,
-			     "SELECT COUNT(DISTINCT %.*s, %.*s) FROM %.*s WHERE (UNIX_TIMESTAMP(expires) = 0) OR (expires > NOW())",
-			     user_col.len, user_col.s,
-			     domain_col.len, domain_col.s,
-			     table.len, table.s);
-    query_str.s = query;
-    if (ul_dbf.raw_query(ul_dbh, &query_str, &res) < 0) {
-	rpc->fault(ctx, 500, "Failed to query AoR count");
-	return;
-    }
+	if (!DB_CAPABILITY(ul_dbf, DB_CAP_RAW_QUERY)) {
+		rpc->fault(ctx, 500, "Database does not support raw queries");
+		return;
+	}
+	if (ul_dbf.use_table(ul_dbh, &table) < 0) {
+		rpc->fault(ctx, 500, "Failed to use table");
+		return;
+	}
 
-    count = (int)VAL_INT(ROW_VALUES(RES_ROWS(res)));
-    ul_dbf.free_result(ul_dbh, res);
+	memset(query, 0, QUERY_LEN);
+	query_str.len = snprintf(query, QUERY_LEN,
+			"SELECT COUNT(DISTINCT %.*s, %.*s) FROM %.*s WHERE (UNIX_TIMESTAMP(expires) = 0) OR (expires > NOW())",
+			ul_user_col.len, ul_user_col.s,
+			ul_domain_col.len, ul_domain_col.s,
+			table.len, table.s);
+	query_str.s = query;
+	if (ul_dbf.raw_query(ul_dbh, &query_str, &res) < 0 || res==NULL) {
+		rpc->fault(ctx, 500, "Failed to query AoR count");
+		return;
+	}
+	if (RES_ROW_N(res) > 0) {
+		count = (int)VAL_INT(ROW_VALUES(RES_ROWS(res)));
+	}
+	ul_dbf.free_result(ul_dbh, res);
 
-    rpc->add(ctx, "d", count);
+	rpc->add(ctx, "d", count);
 }
 
 static const char* ul_rpc_db_users_doc[2] = {
@@ -714,49 +797,51 @@ static const char* ul_rpc_db_users_doc[2] = {
 
 static void ul_rpc_db_contacts(rpc_t* rpc, void* ctx)
 {
-    str table = {0, 0};
-    char query[QUERY_LEN];
-    str query_str;
-    db1_res_t* res;
-    int count;
+	str table = {0, 0};
+	char query[QUERY_LEN];
+	str query_str;
+	db1_res_t* res = NULL;
+	int count = 0;
 
-    if (db_mode == NO_DB) {
-	rpc->fault(ctx, 500, "Command is not supported in db_mode=0");
-	return;
-    }
+	if (ul_db_mode == NO_DB) {
+		rpc->fault(ctx, 500, "Command is not supported in db_mode=0");
+		return;
+	}
 
-    if (rpc->scan(ctx, "S", &table) != 1) {
-	rpc->fault(ctx, 500, "Not enough parameters (table to lookup)");
-	return;
-    }
+	if (rpc->scan(ctx, "S", &table) != 1) {
+		rpc->fault(ctx, 500, "Not enough parameters (table to lookup)");
+		return;
+	}
 
-    if (table.len + 22 > QUERY_LEN) {
-	rpc->fault(ctx, 500, "Too long database query");
-	return;
-    }
+	if (table.len + 22 > QUERY_LEN) {
+		rpc->fault(ctx, 500, "Too long database query");
+		return;
+	}
 
-    if (!DB_CAPABILITY(ul_dbf, DB_CAP_RAW_QUERY)) {
-	rpc->fault(ctx, 500, "Database does not support raw queries");
-	return;
-    }
-    if (ul_dbf.use_table(ul_dbh, &table) < 0) {
-	rpc->fault(ctx, 500, "Failed to use table");
-	return;
-    }
-	
-    memset(query, 0, QUERY_LEN);
-    query_str.len = snprintf(query, QUERY_LEN, "SELECT COUNT(*) FROM %.*s WHERE (UNIX_TIMESTAMP(expires) = 0) OR (expires > NOW())",
-			     table.len, table.s);
-    query_str.s = query;
-    if (ul_dbf.raw_query(ul_dbh, &query_str, &res) < 0) {
-	rpc->fault(ctx, 500, "Failed to query contact count");
-	return;
-    }
+	if (!DB_CAPABILITY(ul_dbf, DB_CAP_RAW_QUERY)) {
+		rpc->fault(ctx, 500, "Database does not support raw queries");
+		return;
+	}
+	if (ul_dbf.use_table(ul_dbh, &table) < 0) {
+		rpc->fault(ctx, 500, "Failed to use table");
+		return;
+	}
 
-    count = (int)VAL_INT(ROW_VALUES(RES_ROWS(res)));
-    ul_dbf.free_result(ul_dbh, res);
+	memset(query, 0, QUERY_LEN);
+	query_str.len = snprintf(query, QUERY_LEN, "SELECT COUNT(*) FROM %.*s WHERE (UNIX_TIMESTAMP(expires) = 0) OR (expires > NOW())",
+			table.len, table.s);
+	query_str.s = query;
+	if (ul_dbf.raw_query(ul_dbh, &query_str, &res) < 0 || res==NULL) {
+		rpc->fault(ctx, 500, "Failed to query contact count");
+		return;
+	}
 
-    rpc->add(ctx, "d", count);
+	if (RES_ROW_N(res) > 0) {
+		count = (int)VAL_INT(ROW_VALUES(RES_ROWS(res)));
+	}
+	ul_dbf.free_result(ul_dbh, res);
+
+	rpc->add(ctx, "d", count);
 }
 
 static const char* ul_rpc_db_contacts_doc[2] = {
@@ -766,49 +851,51 @@ static const char* ul_rpc_db_contacts_doc[2] = {
 
 static void ul_rpc_db_expired_contacts(rpc_t* rpc, void* ctx)
 {
-    str table = {0, 0};
-    char query[QUERY_LEN];
-    str query_str;
-    db1_res_t* res;
-    int count;
+	str table = {0, 0};
+	char query[QUERY_LEN];
+	str query_str;
+	db1_res_t* res = NULL;
+	int count = 0;
 
-    if (db_mode == NO_DB) {
-	rpc->fault(ctx, 500, "Command is not supported in db_mode=0");
-	return;
-    }
+	if (ul_db_mode == NO_DB) {
+		rpc->fault(ctx, 500, "Command is not supported in db_mode=0");
+		return;
+	}
 
-    if (rpc->scan(ctx, "S", &table) != 1) {
-	rpc->fault(ctx, 500, "Not enough parameters (table to lookup)");
-	return;
-    }
+	if (rpc->scan(ctx, "S", &table) != 1) {
+		rpc->fault(ctx, 500, "Not enough parameters (table to lookup)");
+		return;
+	}
 
-    if (table.len + 22 > QUERY_LEN) {
-	rpc->fault(ctx, 500, "Too long database query");
-	return;
-    }
+	if (table.len + 22 > QUERY_LEN) {
+		rpc->fault(ctx, 500, "Too long database query");
+		return;
+	}
 
-    if (!DB_CAPABILITY(ul_dbf, DB_CAP_RAW_QUERY)) {
-	rpc->fault(ctx, 500, "Database does not support raw queries");
-	return;
-    }
-    if (ul_dbf.use_table(ul_dbh, &table) < 0) {
-	rpc->fault(ctx, 500, "Failed to use table");
-	return;
-    }
-	
-    memset(query, 0, QUERY_LEN);
-    query_str.len = snprintf(query, QUERY_LEN, "SELECT COUNT(*) FROM %.*s WHERE (UNIX_TIMESTAMP(expires) > 0) AND (expires <= NOW())",
-			     table.len, table.s);
-    query_str.s = query;
-    if (ul_dbf.raw_query(ul_dbh, &query_str, &res) < 0) {
-	rpc->fault(ctx, 500, "Failed to query contact count");
-	return;
-    }
+	if (!DB_CAPABILITY(ul_dbf, DB_CAP_RAW_QUERY)) {
+		rpc->fault(ctx, 500, "Database does not support raw queries");
+		return;
+	}
+	if (ul_dbf.use_table(ul_dbh, &table) < 0) {
+		rpc->fault(ctx, 500, "Failed to use table");
+		return;
+	}
 
-    count = (int)VAL_INT(ROW_VALUES(RES_ROWS(res)));
-    ul_dbf.free_result(ul_dbh, res);
+	memset(query, 0, QUERY_LEN);
+	query_str.len = snprintf(query, QUERY_LEN, "SELECT COUNT(*) FROM %.*s WHERE (UNIX_TIMESTAMP(expires) > 0) AND (expires <= NOW())",
+			table.len, table.s);
+	query_str.s = query;
+	if (ul_dbf.raw_query(ul_dbh, &query_str, &res) < 0 || res==NULL) {
+		rpc->fault(ctx, 500, "Failed to query contact count");
+		return;
+	}
 
-    rpc->add(ctx, "d", count);
+	if (RES_ROW_N(res) > 0) {
+		count = (int)VAL_INT(ROW_VALUES(RES_ROWS(res)));
+	}
+	ul_dbf.free_result(ul_dbh, res);
+
+	rpc->add(ctx, "d", count);
 }
 
 static const char* ul_rpc_db_expired_contacts_doc[2] = {
